import java.awt.geom.*;
/**
   This class represents a 2ft x 2ft grid location or cell in the 2nd Floor of Nedderman Hall.

 * @author Ashwini Kuntamukkala
 * @version 	%I%, %G%
 * @since       JDK1.2

*/


public class Cell
{
	double x;
	double y;

   /**
   Constructor that takes in point (in inches)

   @param Point
   */

	public Cell(Point2D p)
	{
	 x = Math.floor(p.getX()/12);
	 y = Math.floor(p.getY()/12);
	}
   /**
   Constructor that takes in x and y coordinate
   @param x and y coordinate
   */

	public Cell(double x1, double y1)
	{
	 x = Math.floor(x1/12);
	 y = Math.floor(y1/12);
	}
   /**
   Returns the bounding rectangle for a cell. For example,
   The top left corner of bounding rectangle of cell 1,2 is [12,24]
   whose length and width is 12 pixels.
   @param x and y coordinate
   @return Rectangle2D that is the bounding rectangle for a cell
   */
	public Rectangle2D getBoundingRectangle()
	{
		return new Rectangle2D.Double(x*12,y*12,12,12);
	}

   /**
   Returns the top left coordinate of bounding rectangle for a cell. For example,
   The top left corner of bounding rectangle of cell 1,2 is [12,24]

   @return Rectangle2D that is the bounding rectangle for the grid location
   */

	public Point2D topLeftCoordinate()
	{
		return new Point2D.Double(x*12,y*12);
	}

   /**
   Returns the String representation of a cell. Used for printing purposes. For Example,
   The cell [2,3] is printed as "Cell : [2,3]"

   @return String that represents the grid.
   */

	public String toString()
	{
		return new String("Cell : [" + Double.toString(x) + "," + Double.toString(y) + "]");
	}

   /**
   Returns the X coordinate of a cell. For example,
   The X coordinate of cell [1,2] is 1

   @return X coordinate of a cell
   */

	public int getX() { return (int) x; }
   /**
   Returns the X coordinate of a cell. For example,
   The X coordinate of cell [1,2] is 1.

   @return X coordinate of a cell.
   */
	public int getY() { return (int) y; }

   /**
   Sets the X coordinate of a cell.

   @param  new X coordinate of a cell
   */

	public void setX(double a ) {  x= a; }

   /**
	Sets the Y coordinate of a cell.

    @param new Y coordinate of a cell.

   */

	public void setY(double b ) {  x= b; }

}