import java.awt.*;
import java.awt.geom.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.image.*;
import java.util.*;
import java.io.*;

/**
	This class maintains the graphic display of the current location and orientation
	of the agents and the current wumpus world entities. It enables the user at the
	server to drop the entities in the wumpus world using the mouse.
	It displays the current location of the mouse as it moves over the Wumpus World
	in the status bar.

    @author Ashwini Kuntamukkala
    * @version 	%I%, %G%
    * @since       JDK1.2

*/
class CustomizedPanel extends JPanel
{
	private JLabel statusBar;
	private Vector logOfAllActions;
	private GeneralPath path;
	private Vector[][] objectsAtCell;
	private Dimension size;
	private Vector[] vPoints;
	private Vector[] vObstacles;
	private int nValidPolygons;
	private int nInvalidPolygons;
	private GeneralPath[] validPath;
	private GeneralPath[] invalidPath;
	private String currentSelection;
	private WWServer wwServer;

	private Vector allWumpii;
	private Vector allGold;
	private int nWidth;
	private int nHeight;
	private Environment env;
	private CustomizedPanel() { }

   /**
   Constructor that takes in the Environment, JLabel, plan and obstacle files
   This class represents the Wumpus World graphically over a JPanel
   It reads the plan and obstacle files which model the second floor of Nedderman Hall
   It then divides it into a grid of 2 ft x 2ft which on screen translates to a
   12 pixel x 12 pixel square as 2ft = 12 pixels. This is the scale that is used.

   @param environment, status bar, files for plan and obstacles
   */

	public CustomizedPanel(Environment e, JLabel s, String planFile, String obstacleFile)
	{
		allWumpii = new Vector();
		allGold   = new Vector();
		statusBar = s;
		setOpaque(true);
		fileRead(planFile, obstacleFile);
		constructPlan();
		constructObstacles();
		size = new Dimension(0,0);

		currentSelection = new String("Default");

		objectsAtCell = new Vector[nHeight][nWidth];
		for(int i = 0 ; i < nHeight; i++)
		{
			for(int j = 0; j < nWidth; j++)
			{
				objectsAtCell[i][j] = new Vector();
			}
		}
		env = e;
		env.setAllWumpii(allWumpii);
		env.setAllGold(allGold);
		env.setGridImage(objectsAtCell);

		logOfAllActions = new Vector();

		setBackground(Color.black);
		addMouseMotionListener( new MouseMotionHandler());
		addMouseListener(new MouseClickHandler());

	}
   /**
   Returns the total number cells along Y axis.

   @return Total number of cells along Y axis.
   */

   public int getnHeight()
   {
	   return nHeight;
   }
   /**
   Returns the total number cells along X axis.

   @return total number cells along X axis.
   */
   public int getnWidth()
   {
	   return nWidth;
   }

   /**
   Sets the Environment.

   @param Environment, contains information about the wumpus world entities and agents.
   */

public void setEnvironment(Environment e)
   {
	   env = e;
   }
   /**
   Returns the Environment.

   @return Environment.
   */

   public Environment getEnvironment()
   {
	   return env;
   }

 /**
 	constructs the plan of 2nd floor of nedderman hall from a series of
 	coordinate points that represent corners along the outer boundary.

 */

   public void constructPlan()
   {

	   /*
	      For example
	      points specified are
	      [1,1]
	      [4,1]
	      [4,4]
	      [1,4]

		  then the wumpus world would be like

		[0,0]			X
			.---------------->
			|
		  Y |   ***********
			|   *         *
			|   *         *
			|   *         *
			|   *         *
			|   *         *
			V   ***********

	   */
	   validPath = new GeneralPath[nValidPolygons];

	   int i,j;
	   for(i = 0; i < nValidPolygons; i++)
	   {

		   validPath[i] = new GeneralPath(GeneralPath.WIND_EVEN_ODD);
		   Point2D p = (Point2D) vPoints[i].elementAt(0);
		   Point2D p1;
		   validPath[i].moveTo((float)p.getX(),(float) p.getY());
		   for(j = 1; j < vPoints[i].size(); j++)
		   {
			   p1 = (Point2D) vPoints[i].elementAt(j);
			   validPath[i].lineTo((float)p1.getX(),(float) p1.getY());
		   }
		   validPath[i].closePath();
		   Rectangle r = validPath[i].getBounds();


		   nWidth  = (int)Math.ceil(r.getWidth()/12);
		   nHeight = (int)Math.ceil(r.getHeight()/12);

	   }
   }

    /**
   Sets the Wumpus World Server.

   @param WWServer.
   */

    public void setWWServer(WWServer w)
      {
		  wwServer = w;
	  }
	  public void undoPreviousAction()
	  {

		  if(logOfAllActions.size() > 0)
		  {
			ActionInformation action = (ActionInformation) logOfAllActions.lastElement();
			Cell c = action.getCell();
			logOfAllActions.remove(action);
			objectsAtCell[c.getY()][c.getX()].remove(action.getEntity());
			Object o = action.getEntity();
			if(o instanceof Gold)
			 allGold.remove(o);

			if(o instanceof Wumpus)
			 allWumpii.remove(o);

			repaint();
	  	  }
	  }
    /**
   Sets the pit cursor as crosshair
   */

	  public void setPitCursor()
	  {
		 Image i = Toolkit .getDefaultToolkit() .getImage("pitIcon.gif");
		 Point p = new Point(0,0);
		 currentSelection = new String("Pit");
		 setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
	  }

    /**
   Sets the obstacle cursor as crosshair
   */
	  public void setObstacleCursor()
	  {
		 Image i = Toolkit .getDefaultToolkit() .getImage("ObstacleIcon.gif");
		 Point p = new Point(0,0);
		  currentSelection = new String("Obstacle");
		 setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
	  }
    /**
   Sets the wumpus cursor as crosshair
   */

	  public void setWumpusCursor()
	  {
		 Image i = Toolkit .getDefaultToolkit() .getImage("wumpusFrontIcon.gif");
		 Point p = new Point(0,0);
		  currentSelection = new String("Wumpus");
		 setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
	  }
    /**
   Sets the gold cursor as crosshair
   */

	  public void setGoldCursor()
	  {
		 Image i = Toolkit .getDefaultToolkit() .getImage("pot_o_goldIcon.gif");
		 Point p = new Point(0,0);
		  currentSelection = new String("Gold");
		 setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
  	  }
    /**
   Sets the default cursor
   */

	  public void setDefaultCursor()
	  {
		   currentSelection = new String("Default");
		   setCursor ( new Cursor ( Cursor.DEFAULT_CURSOR) ) ;
	  }
 /**
 	constructs the obstacles in the 2nd floor of nedderman hall from a series of
 	coordinate points.
 */


	public void constructObstacles()
	   {


		/*
		   For Example,
	  	   obstacle file contains
		   [2,2]
		   [3,2]
		   [3,3]
		   [2,3]

		   then the wumpus world would be like

		[0,0]
			.---------------->
			|
			|   **********
			|   *        *
			|   *  ****  *
			|   *  ****  *
			|   *  ****  *
			|   *        *
			V   **********

		    Notice that the obstacles lie inside the plan that was constructed earlier
			by a call to constructPlan

			These points are obtained from a file in the fileRead method
		*/
		   invalidPath = new GeneralPath[nInvalidPolygons];
		   int i,j;
		   for(i = 0; i < nInvalidPolygons; i++)
		   {

			   invalidPath[i] = new GeneralPath(GeneralPath.WIND_EVEN_ODD);
			   Point2D p = (Point2D) vObstacles[i].elementAt(0);
			   Point2D p1;
			   invalidPath[i].moveTo((float)p.getX(),(float) p.getY());
			   for(j = 1; j < vObstacles[i].size(); j++)
			   {
				   p1 = (Point2D) vObstacles[i].elementAt(j);
				   invalidPath[i].lineTo((float)p1.getX(),(float) p1.getY());
			   }
			   invalidPath[i].closePath();
		   }

	   }
		/**
			constructs the series of points that represent the outer boundary of
			2nd Floor of Nedderman hall from a file called "plan.txt" and a series of
			points that represent a boundary of obstacle (a region that cannot be accessed)
			from a file called "obstacle.txt"

			for example the coordinates
			(0'0", 0'0")
			(2'0", 0'0")
			(2'0", 2'0")
			(0'0", 2'0")

			translate to a series of points representing their mapped coordinates on
			the screen. We know that the scale used here is 2ft = 12 pixels
			hence these points are translated to
			(0 , 0)
			(12, 0)
			(12,12)
			(0 ,12)


			the format of "plan.txt" is
			1 => One represents that there is only one outer boundary
			(0'0",3'0")
			(0'5.625",3'0")
			(0'5.625",0'0")
			(12'10",0'0")
			(12'10",3'0")
			.
			.
			V
			there can be multiple obstacles, these are represented by a series of
			points separated by "**************************".

			for example. refer "obstacle.txt"

		 */

		public void fileRead(String planFile, String obstacleFile)
		{

			try{



			FileReader fr = new FileReader(planFile);
			BufferedReader br = new BufferedReader(fr);
			String s;
			String str1 = new String();
			String str2 = new String();
			String str3 = new String();
			String str4 = new String();
			s = br.readLine();

			nValidPolygons = Integer.parseInt(s);
			vPoints = new Vector[Integer.parseInt(s)];
			int k;
			for(k=0;k<nValidPolygons;k++)
				vPoints[k] = new Vector();

			int i = 0;
			StringTokenizer tok;
			while((s=br.readLine())!=null)
			{

				if(s.equals("*************************"))
				{
					i++;
					continue;
				}

				tok = new StringTokenizer(s,"(',\")");
				str1 = tok.nextToken();
				str2 = tok.nextToken();
				str3 = tok.nextToken();
				str4 = tok.nextToken();

				Point2D.Float p = new Point2D.Float(Float.parseFloat(str1)*6+(Float.parseFloat(str2)/2), Float.parseFloat(str3)*6+(Float.parseFloat(str4)/2));
				vPoints[i].add(p);


			}//end outer while


			FileReader fr1 = new FileReader(obstacleFile);
			BufferedReader br1 = new BufferedReader(fr1);
			s = br1.readLine();
			nInvalidPolygons = Integer.parseInt(s);
			vObstacles = new Vector[Integer.parseInt(s)];
			for(k=0;k<nInvalidPolygons;k++)
				vObstacles[k] = new Vector();

			i = 0;

			while((s=br1.readLine())!=null)
			{

				if(s.equals("*************************"))
				{
					i++;
					continue;
				}

				tok = new StringTokenizer(s,"(',\")");
				str1 = tok.nextToken();
				str2 = tok.nextToken();
				str3 = tok.nextToken();
				str4 = tok.nextToken();
				Point2D.Float p = new Point2D.Float(Float.parseFloat(str1)*6+(Float.parseFloat(str2)/2), Float.parseFloat(str3)*6+(Float.parseFloat(str4)/2));
				vObstacles[i].add(p);
			}//end outer while

			fr.close();
		    fr1.close();
			}
			catch(Exception e)
			{
				System.out.println("Error occurred");
			}
	    }
/**
	This method draws the 2nd Floor of Nedderman Hall over the JPanel and
	the spits it into a grid of 12 x 12 pixels. for each cell, it displays the
	Wumpus World Entity that is present in it. It displays the current location
	and orientation of the agents. The agents are represented as triangles within
	the cell. The pointed edge represents their current direction of orientation.
	^ represents facing north
	> represents facing east
	< represents facing west
	V represents facing south.

	When the scheme is FCFS then
	If the agent is waiting for other agents to finish the current trial after it has
	finished its current trial, it is
	shown in "CYAN" color else in "MAGENTA"

	When the scheme is Round Robin then
	If the agent has obtained connection and is waiting for Server to stop accepting
	connection so that the trial can start or when the agent is waiting for other agents
	to finish current trial when the agent has itself finished the trial, then the
	agent is displayed in "CYAN" else in "MAGENTA"
*/

	protected synchronized void paintComponent(Graphics g)
	{
		super.paintComponent(g);

	Graphics2D g2d = (Graphics2D) g;
	BufferedImage buffImage = new BufferedImage(12,12, BufferedImage.TYPE_INT_RGB);
	Graphics2D gg = buffImage.createGraphics();

	gg.setColor(Color.black);
	gg.setColor(Color.white);
	gg.fillRect(0,0,12,12);
    gg.setColor(Color.black);
    gg.drawRect(0,0,12,12);

	g2d.setColor(Color.white);
	g2d.setPaint(
		new TexturePaint(buffImage, new Rectangle(12,12))
		);
	int i;
	for(i = 0; i < nValidPolygons ; i++)
	{
		g2d.fill(validPath[i]);
	}

	g2d.setColor(Color.black);
	for(i = 0; i < nInvalidPolygons ; i++)
	{
		g2d.fill(invalidPath[i]);
	}

	for(int p = 0; p < nHeight; p++)
	{
		for(int q = 0; q < nWidth; q++)
		{
			for(int l = 0; l < objectsAtCell[p][q].size(); l++)
			{
				Object o = objectsAtCell[p][q].elementAt(l);
				if (o instanceof Pit)
				{
					g2d.setColor(Color.red);
					g2d.fillRect(q*12, p*12, 6,6);
				}
				else
				{
					 if(o instanceof Obstacle)
					 {
					 	g2d.setColor(Color.blue);
					 	g2d.fillRect((q*12)+6, (p*12), 6,6);
					 }
					 else
					 {
					  if(o instanceof Wumpus)
					  {
					 	g2d.setColor(Color.green);
					 	g2d.fillRect((q*12), (p*12)+6, 6,6);
					  }
					  else
					  {
					 	g2d.setColor(Color.lightGray);
					 	g2d.fillRect((q*12)+6, (p*12)+6, 6,6);
					  }
					}
			   }

			}
		}

		//
		if(wwServer!=null)
		{
			if(wwServer.isSchemeRR())
			{
				Vector agentList = wwServer.getAgentList();
				if(agentList!=null)
				{
					for(int k=0; k<agentList.size();k++)
					{
						MovingObject mo = null;
						mo = ((WWServer.RoundRobinAgent) agentList.elementAt(k)).getCurrentLocation();
						if(((WWServer.RoundRobinAgent) agentList.elementAt(k)).isTrialOver())
						{
							// Agent is waiting for other agents to finish the trial
							g2d.setColor(Color.cyan);
						}
						else
						{
							// Agent is playing the trial
							g2d.setColor(Color.magenta);
						}

						if(mo!=null)
						{
							int[] x = new int[3];
							int[] y = new int[3];

							switch(mo.getOrientation())
							{
								case Directions.NORTH:
								{
									x[0] = mo.getLocation().getPtX()*12+6;
									x[1] = mo.getLocation().getPtX()*12+12;
									x[2] = mo.getLocation().getPtX()*12;

									y[0] = mo.getLocation().getPtY()*12;
									y[1] = mo.getLocation().getPtY()*12+12;
									y[2] = mo.getLocation().getPtY()*12+12;
									break;
								}
								case Directions.SOUTH:
								{
									x[0] = mo.getLocation().getPtX()*12+6;
									x[1] = mo.getLocation().getPtX()*12;
									x[2] = mo.getLocation().getPtX()*12+12;

									y[0] = mo.getLocation().getPtY()*12+12;
									y[1] = mo.getLocation().getPtY()*12;
									y[2] = mo.getLocation().getPtY()*12;

									break;
								}
								case Directions.EAST:
								{
									x[0] = mo.getLocation().getPtX()*12+12;
									x[1] = mo.getLocation().getPtX()*12;
									x[2] = mo.getLocation().getPtX()*12;

									y[0] = mo.getLocation().getPtY()*12+6;
									y[1] = mo.getLocation().getPtY()*12+12;
									y[2] = mo.getLocation().getPtY()*12;

									break;
								}
								case Directions.WEST:
								{
									x[0] = mo.getLocation().getPtX()*12;
									x[1] = mo.getLocation().getPtX()*12+12;
									x[2] = mo.getLocation().getPtX()*12+12;

									y[0] = mo.getLocation().getPtY()*12+6;
									y[1] = mo.getLocation().getPtY()*12;
									y[2] = mo.getLocation().getPtY()*12+12;

									break;
								}

							} // end of switch case
							g2d.fillPolygon(x,y,3);
						} // end of if statement


					}
				}

			}
			else
			{
				Vector agentThreads = wwServer.getAgentThreads();
				if(agentThreads!=null)
				{
					for(int t = 0; t < agentThreads.size(); t++ )
					{
						MovingObject mo = null;
						 mo = ((WWServer.AgentThread) agentThreads.elementAt(t)).getCurrentLocation();
						if(((WWServer.AgentThread) agentThreads.elementAt(t)).isTrialOver())
						{
							// Agent is waiting for other agents to finish the trial
							g2d.setColor(Color.cyan);
						}
						else
						{
							// Agent is playing the trial
							g2d.setColor(Color.magenta);
						}

						if(mo!=null)
						{
							int[] x = new int[3];
							int[] y = new int[3];

							switch(mo.getOrientation())
							{
								case Directions.NORTH:
								{
									x[0] = mo.getLocation().getPtX()*12+6;
									x[1] = mo.getLocation().getPtX()*12+12;
									x[2] = mo.getLocation().getPtX()*12;

									y[0] = mo.getLocation().getPtY()*12;
									y[1] = mo.getLocation().getPtY()*12+12;
									y[2] = mo.getLocation().getPtY()*12+12;
									break;
								}
								case Directions.SOUTH:
								{
									x[0] = mo.getLocation().getPtX()*12+6;
									x[1] = mo.getLocation().getPtX()*12;
									x[2] = mo.getLocation().getPtX()*12+12;

									y[0] = mo.getLocation().getPtY()*12+12;
									y[1] = mo.getLocation().getPtY()*12;
									y[2] = mo.getLocation().getPtY()*12;

									break;
								}
								case Directions.EAST:
								{
									x[0] = mo.getLocation().getPtX()*12+12;
									x[1] = mo.getLocation().getPtX()*12;
									x[2] = mo.getLocation().getPtX()*12;

									y[0] = mo.getLocation().getPtY()*12+6;
									y[1] = mo.getLocation().getPtY()*12+12;
									y[2] = mo.getLocation().getPtY()*12;

									break;
								}
								case Directions.WEST:
								{
									x[0] = mo.getLocation().getPtX()*12;
									x[1] = mo.getLocation().getPtX()*12+12;
									x[2] = mo.getLocation().getPtX()*12+12;

									y[0] = mo.getLocation().getPtY()*12+6;
									y[1] = mo.getLocation().getPtY()*12;
									y[2] = mo.getLocation().getPtY()*12+12;

									break;
								}

							} // end of switch case
							g2d.fillPolygon(x,y,3);
							//g2d.fillOval(mo.getLocation().getPtX()*12+1, mo.getLocation().getPtY()*12 +1, 10,10);
						} // end of if statement
					} // end of for statement
				} // end of if statement
			} // end of else statement
		}
		g2d.setColor(Color.white);
		g2d.drawString("Legend : ", 45, 90);

		g2d.fillRect(40,95,85,75);
		g2d.setColor(Color.red);
		g2d.fillRect(45,100,6,6);
		g2d.drawString(" : Pit", 55,106);
		g2d.setColor(Color.green);
		g2d.fillRect(45,115,6,6);
		g2d.drawString(" : Wumpus", 55,121);
		g2d.setColor(Color.gray);
		g2d.fillRect(45,130,6,6);
		g2d.drawString(" : Gold", 55,136);
		g2d.setColor(Color.blue);
		g2d.fillRect(45,145,6,6);
		g2d.drawString(" : Obstacle", 55,151);

	}

	scrollRectToVisible(new Rectangle(0,0,2532,2532));
	revalidate();
	}

/**
	Causes the panel to paint itself again.
*/
	public void draw()
	{
		repaint();
	}

/**
	Returns if the point is a valid point or not.
	A point is valid if it is within the Wumpus World

	@param point that needs to be tested for validity
	@return true or false
*/
	public boolean isValidPoint(Point2D p)
	{
		int i;
		boolean validFlag = false;
		for(i=0;i<validPath.length;i++)
		{
			if(validPath[i].contains(p))
			{
				validFlag = true;
				break;
			}
		}
		boolean invalidFlag = false;
		for(i=0;i<invalidPath.length;i++)
		{
			if(invalidPath[i].contains(p))
			{
				invalidFlag = true;
				break;
			}
		}
		boolean flag = false;
		if(validFlag==true)
		{
			if(invalidFlag==true)
			{
				flag = false;
			}
			else
			{
				flag = true;
			}
		}
		return flag;
	}
/**
	This method reads the world from a file and loads the wumpus world entities
	in cells specified in the file.
	@param Name of the file
*/
	public void readWorld(String s)
	{

		try
		{
		  FileInputStream fin =  new FileInputStream(s);

		  // now turn the FileInputStream into a DataInputStream
		  try
		  {
			java.io.DataInputStream myInput = new java.io.DataInputStream(fin);
			String thisLine=null;
   			StringTokenizer tokenizer;
			try
			{
			  thisLine = myInput.readLine();
			  if(thisLine==null)
			  	return;
			  else
			  {
				tokenizer = new StringTokenizer(thisLine, "[,] ");
				nHeight = Integer.parseInt(tokenizer.nextToken());
				nWidth  = Integer.parseInt(tokenizer.nextToken());

				logOfAllActions = new Vector();
				objectsAtCell = new Vector[nHeight][nWidth];
				for(int i = 0; i < nHeight; i++)
				{
					for(int j = 0; j < nWidth; j++)
					{
						objectsAtCell[i][j] = new Vector();
					}
				}
			  }

			   /* Reset the vector for all wumpii */
			   allWumpii = new Vector();

			   /* Sets the grid image and all wumpii vector in environment */
			   env.setGridImage(objectsAtCell);
			   env.setAllWumpii(allWumpii);

			  while ((thisLine = myInput.readLine()) != null)
			   {
				tokenizer = new StringTokenizer(thisLine, "Cell [,]:");

				int x = Integer.parseInt(tokenizer.nextToken());
				int y = Integer.parseInt(tokenizer.nextToken());
				String entity = tokenizer.nextToken();

				if(entity.equals("P"))
				{
					Pit  pit = new Pit();
					logOfAllActions.add(new ActionInformation(new Cell(x*12,y*12), pit ));
					objectsAtCell[y][x].add(pit);
				}
				else
				if(entity.equals("O"))
				{
					Obstacle obstacle = new Obstacle();
					logOfAllActions.add(new ActionInformation(new Cell(x*12,y*12), obstacle));
					objectsAtCell[y][x].add(obstacle);
				}
				else
				if(entity.equals("G"))
				{
					Gold gold = new Gold();
					logOfAllActions.add(new ActionInformation(new Cell(x*12,y*12), gold));
					objectsAtCell[y][x].add(gold);

				}
				else
				{
					Wumpus wumpus = new Wumpus(1, x, y, 3, 0, "W", tokenizer.nextToken() );
					logOfAllActions.add(new ActionInformation(new Cell(x*12,y*12), wumpus));
					objectsAtCell[y][x].add(wumpus);
					allWumpii.add(wumpus);
				}

			   } // while loop ends here

			}
			catch (Exception x1)
			{
			 System.err.println("Error: " + x1);
			}
		  } // end try
		  catch (Exception x2)
		  {
			System.err.println("Error: " + x2);
		  }

		 } // end try
		 catch (Exception x3)
		 {
		  System.err.println("failed to open file output");
		  System.err.println("Error: " + x3);
		 }

 	repaint();
 	}

 	/**
 		this method writes the world to the file specified here.
 		@param Name of file
 	*/
	public void writeWorld(String s)
	{
		try
		{
			FileWriter fw = new FileWriter(s);
			fw.write("[ " + Integer.toString(nHeight) + "," + Integer.toString(nWidth) + " ]" + '\n');
			for(int i = 0; i < nHeight; i++)
			{
				for(int j = 0; j < nWidth; j++)
				{

					for(int l=0;l<objectsAtCell[i][j].size();l++)
					{
						Object o = objectsAtCell[i][j].elementAt(l);
						if (o instanceof Pit)
						{
						   fw.write("Cell[" + j + "," + i + "] : P " + '\n');
						}
						else
						{
							 if(o instanceof Obstacle)
							 {
					 			fw.write("Cell[" + j + "," + i + "] : O " + '\n');
							 }
							 else
							 {
							  if(o instanceof Wumpus)
							  {
								Wumpus w = (Wumpus) o;
								int mode = w.getMode();
								String sMode = new String("");
								if(mode == WumpusMode.STAY)
								 sMode = new String("STAY");

								if(mode == WumpusMode.SPIN)
								 sMode = new String("SPIN");
								if(mode == WumpusMode.WALK)
								 sMode = new String("WALK");
								if(mode == WumpusMode.GOTO_GOLD)
								 sMode = new String("GOTO_GOLD");
								if(mode == WumpusMode.GOTO_PIT)
								 sMode = new String("GOTO_PIT");
								if(mode == WumpusMode.RANDOM)
								 sMode = new String("RANDOM");

					  			fw.write("Cell[" + j + "," + i + "] : W " + sMode + '\n');
							  }
							  else
							  {
					  			fw.write("Cell[" + j + "," + i + "] : G " + '\n');
							  }
							}
					   }

					}
				}
			}
			fw.close();
		}
		catch(java.io.IOException e)
		{
			System.out.println("Exception while writing to file");
		}

	}
/**
	This methods finds out if a cell is completely within the wumpus world or not.
	Since we are modelling 2nd Floor of Nedderman hall, several cells may not be
	completely within the Wumpus World.
*/
	public boolean isCompletelyInside(Cell c)
	{
		int i ;
		Rectangle2D r = c.getBoundingRectangle();

		boolean validFlag = false;
		boolean invalidFlag = false;

		for(i = 0; i < validPath.length; i++ )
		{
			if(validPath[i].contains(r))
			{
				validFlag = true;
				break;
			}
			else
			{
				validFlag = false;
				break;
			}

		}


		for(i=0;i<invalidPath.length;i++)
		{
			if(invalidPath[i].intersects(r))
			{
				invalidFlag = true;
				break;
			}
		}
		boolean flag = false;
		if(validFlag)
		{
			if(invalidFlag)
				flag = false;
			else
				flag = true;
		}
		return flag;
	}
/**
	Displays an error message when a pit is put at a place where there is already an obstacle or another pit.
*/
	private void displayErrorMsg(String s)
	{

		MessageDialog x = new MessageDialog(new JFrame(s), true, "Cannot put pit here. There is either an obstacle or another pit already");
		x.show();
	}
/**
	This private class shows the contents of the current cell upon double clicking the mouse.
	It also puts the wumpus, pit, obstacle, gold into the Wumpus World environment upon
	droping them on a valid cell ( a cell that is completely within in the WW) using the mouse.
*/
	private class MouseClickHandler extends MouseAdapter
	{


		public MouseClickHandler()
		{

		}
		public void mouseReleased(MouseEvent e)
		{
			if(e.getClickCount()>1)
			{
				Point2D.Float p = new Point2D.Float(e.getX(), e.getY());
				if(isValidPoint(p))
				{
					Cell dummy = new Cell(p);
					if(isCompletelyInside(dummy))
					{
						GridInfoDialog gid = new GridInfoDialog(new JFrame(), true,dummy, objectsAtCell[dummy.getY()][dummy.getX()] );
						gid.show();
					}
				}
			}
		}



	    public void mouseClicked(MouseEvent e)
		{
			Point2D.Float p = new Point2D.Float(e.getX(), e.getY());
			if(isValidPoint(p))
			{
				Cell dummy = new Cell(p);
				if(isCompletelyInside(dummy))
				{
					statusBar.setText("Clicked at " + dummy.toString());
					if(currentSelection.equals("Pit"))
					{
						/*
						Add Pit at the cell if and only if
						 there is no pit and no obstacle
						*/
						if(!env.isObstacleAt(dummy.getX(), dummy.getY()) && !env.isPitAt(dummy.getX(), dummy.getY()))
						{
							Pit  pit = new Pit();
							logOfAllActions.add(new ActionInformation(dummy, pit ));
							objectsAtCell[dummy.getY()][dummy.getX()].add(pit);
							repaint();
						}
						else
						{

							MessageDialog x = new MessageDialog(new JFrame(), true, "Cannot put pit here. There is either an obstacle or another pit already in this cell");
							x.show();
						}
					}
					else
					 if(currentSelection.equals("Obstacle"))
					 {
						 if(!env.isPitAt(dummy.getX(), dummy.getY())&&!env.isObstacleAt(dummy.getX(), dummy.getY())&&!env.isWumpusAt(dummy.getX(), dummy.getY())&&!env.isGoldAt(dummy.getX(), dummy.getY()))
						 {
							Obstacle obstacle = new Obstacle();
							logOfAllActions.add(new ActionInformation(dummy, obstacle));
						 	objectsAtCell[dummy.getY()][dummy.getX()].add(obstacle);
						 	repaint();
					 	 }
					 	 else
					 	 {
							MessageDialog x = new MessageDialog(new JFrame(), true, "Cannot put obstacle here. There is either an obstacle, a pit, some gold or a wumpus already in this cell");
							x.show();
						 }
					 }
					 else
					  if(currentSelection.equals("Gold"))
					  {
						  if(!env.isObstacleAt(dummy.getX(), dummy.getY())&&!env.isGoldAt(dummy.getX(), dummy.getY()))
						  {
							  Gold gold = new Gold();
							  logOfAllActions.add(new ActionInformation(dummy, gold));
							  env.addGoldAt(new GridPoint(dummy.getX(), dummy.getY()));
						  	  objectsAtCell[dummy.getY()][dummy.getX()].add(gold);
						  	  repaint();
					  	  }
					  	  else
					  	  {
							MessageDialog x = new MessageDialog(new JFrame(), true, "Cannot put gold here. There is either an obstacle or some gold already in this cell");
							x.show();
						  }
					  }
					  else
					   if(currentSelection.equals("Wumpus"))
					    {
							if(!env.isObstacleAt(dummy.getX(), dummy.getY())&&!env.isWumpusAt(dummy.getX(), dummy.getY()))
							{




							    wwServer.getWumpusMode();

							   Wumpus wumpus = new Wumpus(1,dummy.getX(), dummy.getY(), 3, 0, "Wumpus", wwServer.getSelWumpusMode() );

							  logOfAllActions.add(new ActionInformation(dummy, wumpus));
							  objectsAtCell[dummy.getY()][dummy.getX()].add(wumpus);
							  allWumpii.add(wumpus);
							  repaint();
							}
							else
							{
							MessageDialog x = new MessageDialog(new JFrame(), true, "Cannot put wumpus here. There is either an obstacle or a wumpus already in this cell");
							x.show();

							}
					    }
				}
				else
				{
					statusBar.setText("Clicked at " + dummy.toString() + " : Not completely within the wumpus world");
				}

			}
			else
			{
				statusBar.setText("Clicked at invalid location" );
			}

		}

		public void mouseEntered(MouseEvent e)
		{
			statusBar.setText("Mouse in window..");
		}

		public void mouseExited(MouseEvent e)
		{
			statusBar.setText("Mouse outside the window..");
		}
	}
/**
	Displays the current position of the mouse over the model of 2nd floor of Nedderman Hall
	as it moves on the panel.
*/
	private class MouseMotionHandler extends MouseMotionAdapter
	{
		public void mouseMoved(MouseEvent e)
		{
			Point2D.Float p = new Point2D.Float(e.getX(), e.getY());
			if(isValidPoint(p))
			{
				Cell dummy = new Cell(p);
				if(isCompletelyInside(dummy))
				{
					statusBar.setText("Moving over " + dummy.toString());
				}
				else
				{
					statusBar.setText("Moving over " + dummy.toString() + " : Not completely within the wumpus world");
				}

			}
			else
			{
				statusBar.setText("Moving over invalid location" );
			}
		}
	}


}

/**
	this class is a dialog that displays the error message when the
	wumpus world entities are being dropped onto the Wumpus World
*/
class MessageDialog extends JDialog
{
	JPanel center, panel;
	private MessageDialog() { }
	public MessageDialog(JFrame frame, boolean doModal, String s)
	{
		super(frame, doModal);
		getContentPane().setLayout(new BorderLayout());
		JTextField jTF = new JTextField(s);
		jTF.setEditable(false);
		getContentPane().add(jTF);
		setSize(600,70);
	}

}

/**
	This class displays the contents of the cell where the mouse was double clicked.
*/
class GridInfoDialog extends JDialog
{
	private GridInfoDialog() { }
	public GridInfoDialog(JFrame frame, boolean doModal, Cell c, Vector v)
	{

		super(frame, doModal);
		JTextArea jTextArea = new JTextArea();
		jTextArea.setEditable(false);
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(new JScrollPane(jTextArea),BorderLayout.CENTER);

		JTextField jTextField = new JTextField("Content of " + c);
		jTextField.setEditable(false);
		getContentPane().add(jTextField, BorderLayout.NORTH);

		for(int i = 0; i < v.size();i++)
		{
			Object o = v.elementAt(i);
			if (o instanceof Pit)
			{
			   jTextArea.append("Contains Pit\n");
			}
			else
			{
				 if(o instanceof Obstacle)
				 {
					jTextArea.append("Has An Obstacle\n");
				 }
				 else
				 {
				  if(o instanceof Wumpus)
				  {
					jTextArea.append("Careful...has a wumpus\n");
					Wumpus w = (Wumpus) v.elementAt(i);
					int orientation = w.getOrientation();
					switch(orientation)
					{
						case 2:
							jTextArea.append("Wumpus is facing EAST");
							break;
						case 1:
							jTextArea.append("Wumpus is facing NORTH");
							break;
						case 4:
							jTextArea.append("Wumpus is facing WEST");
							break;
						case 3:
							jTextArea.append("Wumpus is facing SOUTH");
							break;
					}
					jTextArea.append(" and its mode is " + WumpusMode.toString(w.getMode()) + "\n");
				  }
				  else
				  {
					if(o instanceof Gold)
						jTextArea.append("Wow!! There is gold!\n");
				  }
				}
		   }

		}
		setSize(300,300);
	}
}