import java.util.Vector;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.io.*;
import java.util.Random;

/**
   This class contains the information about the objects and agents
   that are contained within the wumpus world grid. It provides methods
   to retrieve and manipulate the information contained within it.

   @author Kruti Mehta
 * @version 	%I%, %G%
 * @since       JDK1.2

*/

public class Environment
{
   /**
   Represents the size of the world described as size x size.
   */
   private int size;

   /**
   Represents the environment consisting of gold pieces, wumpii, pits
   and obstacles in a data structure representing a grid.
   */
   Vector[][] gridImage;

   /**
   A vector that carries a reference to all the wumpus in the world.
   */
   private Vector allWumpii;
   private Vector allGold;

   private Vector allAgents;

   private CustomizedPanel cp;

   private Random randGen;

   private Vector allRRAgents;
   private WWServer wwServer;
   /**
   Constructor
   */
   public Environment()
   {
    allWumpii = new Vector(3);
    allGold = new Vector();
    randGen = new Random();
    allAgents = new Vector();
    allRRAgents = new Vector();

   }

   public void setWWServer(WWServer w)
   {
	   wwServer = w;
   }
   /**
   Returns the dimension of the world.
   @return worldsize.
   */
   public int getSize()
   {
   return size;
   }

   /**
   Returns the gridImage representing the Wumpus World environment.
   @return wumpus world.
   */
   public  Vector[][] getGridImage()
   {
     return gridImage;
   }

   /**
   	Returns allAgents
   	@return Vector allAgents
   */

   public Vector getAllAgents()
   {
	   return allAgents;
   }


   public void setGridImage(Vector objectsInCell[][])
   {
	   gridImage = objectsInCell;
   }

   /**
   	Returns the number of cells along X direction
   	@return number of cells along X direction
   */

   public int getWidth()
   {
	   return gridImage[0].length;
   }
   /**
   	Returns the number of cells along Y direction
   	@return number of cells along Y direction
   */

   public int getLength()
   {
	   return gridImage.length;
   }

    /**
		Sets allAgents to the list of agents being served in a round robin fashion
		@param list of agents being served in a round robin fashion
   */
  public void setAllRRAgents(Vector aA)
   {
	   allRRAgents = aA;
   }

    /**
		Sets allAgents to the list of agents being served in a FCFS fashion
		@param list of agents being served in a FCFS fashion
   */

   public void setAllAgents(Vector aA)
   {
	   allAgents = aA;
   }

    /**
		Returns a list of all wumpii in the Wumpus World
		@return all wumpii
   */

   public Vector getAllWumpii()
   {
	   return allWumpii;
   }

    /**
		Returns a list of all gold treasures in the Wumpus World
		@return all Gold

   */

   public Vector getAllGold()
   {
	   return allGold;
   }

    /**
		Sets allGold to the list of gold treasures in Wumpus World
		@param list of gold treasures in Wumpus World
   */

   public void setAllGold(Vector g)
   {
	   allGold = g;
   }

    /**
   Removes gold treasure from the given location.
   @param location and position
   @return TRUE if successful, FALSE otherwise.
   */

	public void removeGoldFrom(GridPoint gp)
	{
		if(allGold!=null)
		{
			int len = allGold.size();
			for(int i=0; i < len; i++)
			{
				GridPoint g = (GridPoint) allGold.elementAt(i);
				if(g.equals(gp.clone()))
				{
					allGold.removeElementAt(i);
				}
			}
		}

	}

   /**
   Removes agent from the given location.
   @param location and position
   @return TRUE if successful, FALSE otherwise.
   */

    public void removeAgentFrom(GridPoint gp)
    {
		if(allAgents != null)
		{
			int len = allAgents.size();
			for(int i=0;i<len;i++)
			{
				GridPoint g = (GridPoint) allAgents.elementAt(i);
				if(g.equals(gp.clone()))
				{
					allAgents.removeElementAt(i);
				}
			}
		}
	}

   /**
   	Adds an agent at a specified location
   	@param gridpoint
   */

	public void addAgentAt(GridPoint gp)
	{
		if(allAgents!=null)
		{
			allAgents.add(gp.clone());
		}
	}

   /**
   	Adds the gold at a specified location
   	@param gridpoint
   */

   public void addGoldAt(GridPoint gp)
   {
		if(allGold!=null)
		{
			allGold.add(gp.clone());
		}
   }
   /**
   	Sets list of all wumpii
   	@param list of all wumpii
   */

   public void setAllWumpii(Vector aw)
   {
	   allWumpii = aw;
   }

   /**
    Sets CustomizedPanel
   	@param CustomizedPanel
   */

   public void setCustPanel(CustomizedPanel x)
   {
	   cp = x;
   }

   /**
   	Returns CustomizedPanel
   	@return CustomizedPanel
   */
   public CustomizedPanel getCustPanel()
   {
	   return cp;
   }

   /**
   Determines whether a given gridpoint is within the bounds of the world.
   @param gridpoint to valid.
   @return TRUE if gridpoint within bounds, FALSE otherwise.
   */
   public synchronized boolean isValid(GridPoint gp)
   {
    boolean valid = false;
    int gx = gp.getPtX();
    int gy = gp.getPtY();

    if((gx<0) || (gy<0))
      {
        valid = false;
        return valid;
      }

    Cell c = new Cell(gx*12,gy*12);

	if(cp.isCompletelyInside(c) && !isObstacleAt(gx,gy))
	{
		System.out.println("" + gx + "," + gy + "is Valid" );
		valid = true;
    }
    else
    {
	   valid = false;
       System.out.println("" + gx + "," + gy + "is Invalid" );
    }

    return valid;
   }

   /**
   Determines if a grid location is deadly for any agent entering it.
   A location is deadly if there is a wumpus or a pit at that location.
   @param Gridpoint for checking
   @return TRUE if wumpus or pit is at the gridpoint, FALSE otherwise.
   */
   public synchronized boolean isDeadly(GridPoint gp)
   {
    boolean deadly = false;
    //Cell c = new Cell(gp.getPtX(),gp.getPtY());
    if(isWumpusAt(gp.getPtX(),gp.getPtY()) || isPitAt(gp.getPtX(),gp.getPtY()))  //CHANGE ENV.ISWUMPUSAT TO TAKE GRIDPOINT AS PARAMETER.
      {
        deadly= true;
      }
    return deadly;
   }

   /**
   Returns the vector of objects at grid location (x, y).
   @param x and y parameters of the location
   @return Vector of objects at location (x,y).
   */
   public synchronized Vector getObjectAtLocation(int x, int y)
   {
    Vector v = new Vector();
    if(gridImage[y][x] instanceof Vector)
    {
     v = gridImage[y][x];
    }
    else
      System.out.println("EMPTY VECTOR BEING RETURNED");
    return v;
   }



   /**
   Sets targets for wumpii which have a mode that requires them to
   move to other locations with specific properties. For e.g. wumpus
   with mode GO_TO gold requires a target location with gold in it.
   */
   public synchronized void setWumpusTargets()
   {

    Vector goldLocations = new Vector();
    Vector pitLocations = new Vector();

    for(int i=0; i < cp.getnHeight(); ++i)
    {
     for(int j=0; j < cp.getnWidth(); ++j)
     {
      GridPoint gp = new GridPoint(j,i);
      //Cell c = new Cell(i,j);
      if(isGoldAt(j,i))
      {
       goldLocations.addElement(gp);
      }
      if(isPitAt(gp.getPtX(),gp.getPtY()))
      {

       pitLocations.addElement(gp);
      }
     }
    }


    for(int i = 0; i < allWumpii.size(); ++i)
    {
     Wumpus wumpusAtI = (Wumpus) allWumpii.elementAt(i);
 	   if(wumpusAtI.getMode()==WumpusMode.WALK)
 	   {
		    	GridPoint wumpLoc = wumpusAtI.getLocation();
		        wumpusAtI.setTargetLoc(wumpusAtI.getLocation());

	   }
	   if(wumpusAtI.getMode()==WumpusMode.RANDOM)
	   {
		    	GridPoint wumpLoc = wumpusAtI.getLocation();
		        wumpusAtI.setTargetLoc(wumpusAtI.getLocation());
	   }
	   if(wumpusAtI.getMode()==WumpusMode.SPIN)
	   {
		    	GridPoint wumpLoc = wumpusAtI.getLocation();
		        wumpusAtI.setTargetLoc(wumpusAtI.getLocation());
	   }
     }

   }

     /**
     Returns if there is an agent who has a corresponding
     Human avatar at a specified gridpoint
     This method is used when FCFS scheme is followed for handling
     agents is being used.
     @param Gridpoint
     @return true if such an agent exists else false
     */
   public synchronized boolean isAgentWithHAt(GridPoint gp)
   {
	if(!isValid(gp))
	 return false;

    boolean agentExists = false;

    Vector agentThreads = wwServer.getAgentThreads();

	if(agentThreads!=null)
	{
		for(int t = 0; t < agentThreads.size(); t++ )
		{
			WWServer.AgentThread x = (WWServer.AgentThread) agentThreads.elementAt(t);
			MovingObject mo = x.getCurrentLocation();
			if(mo!=null)
			{
				if(gp.getPtX() == mo.getLocation().getPtX() && gp.getPtY() == mo.getLocation().getPtY())
				{
					if(x.getAvataar()!=null)
						if(x.getAvataar().equals("Human"))
						{
				 			agentExists = true;
						}
					break;
			 	}
			}
		}
	}

     return agentExists;

   }
     /**
     Returns if there is an agent who has a corresponding
     Robot avatar at a specified gridpoint
     This method is used when FCFS scheme is followed for handling
     agents is being used.
     @param Gridpoint
     @return true if such an agent exists else false
     */
   public synchronized boolean isAgentWithRAt(GridPoint gp)
   {

	Vector agentThreads = wwServer.getAgentThreads();

	if(!isValid(gp))
	 return false;

    boolean agentExists = false;



	if(agentThreads!=null)
	{
		for(int t = 0; t < agentThreads.size(); t++ )
		{
			WWServer.AgentThread x = (WWServer.AgentThread) agentThreads.elementAt(t);
			MovingObject mo = x.getCurrentLocation();

			if(mo!=null)
			{
				if(gp.getPtX() == mo.getLocation().getPtX() && gp.getPtY() == mo.getLocation().getPtY())
				{
					System.out.println(x.getAvataar());
					if(x.getAvataar()!=null)
						if(x.getAvataar().equals("Robot"))
						{
				 			agentExists = true;
						}
					break;
			 	}
			}
		}
	}
	else
	{

	}


     return agentExists;

   }

     /**
     Returns if there is an agent who has a corresponding
     physical avatar (robot or human) at a specified gridpoint
     This method is used when FCFS scheme is followed for handling
     agents is being used.
     @param Gridpoint
     @return true if such an agent exists else false
     */
   public synchronized boolean isAgentWithR_HAt(GridPoint gp)
   {

	Vector agentThreads = wwServer.getAgentThreads();
	if(!isValid(gp))
	 return false;

    boolean agentExists = false;



	if(agentThreads!=null)
	{
		for(int t = 0; t < agentThreads.size(); t++ )
		{
			WWServer.AgentThread x = (WWServer.AgentThread) agentThreads.elementAt(t);
			MovingObject mo = x.getCurrentLocation();
			if(mo!=null)
			{
				if(gp.getPtX() == mo.getLocation().getPtX() && gp.getPtY() == mo.getLocation().getPtY())
				{
					if(x.getAvataar()!=null)
						if(x.getAvataar().equals("Robot") || x.getAvataar().equals("Human"))
						{
				 			agentExists = true;
						}
					break;
			 	}
			}
		}
	}

     return agentExists;

   }

     /**
     Returns if there is an agent who has a corresponding
     Human avatar (robot or human) at a specified gridpoint
     This method is used when the round robin scheme for handling
     agents is being used.
     @param Gridpoint
     @return true if such an agent is found else false
     */

   public synchronized boolean isRRAgentWithHAt(GridPoint gp)
   {
	if(!isValid(gp))
	 return false;

    boolean agentExists = false;

    allRRAgents = wwServer.getAgentList();

	if(allRRAgents!=null)
	{
		for(int t = 0; t < allRRAgents.size(); t++ )
		{
			WWServer.RoundRobinAgent x = (WWServer.RoundRobinAgent) allRRAgents.elementAt(t);
			MovingObject mo = x.getCurrentLocation();
			if(mo!=null)
			{
				if(gp.getPtX() == mo.getLocation().getPtX() && gp.getPtY() == mo.getLocation().getPtY())
				{
					if(x.getAvataar()!=null)
						if(x.getAvataar().equals("Human"))
						{
				 			agentExists = true;
						}
					break;
			 	}
			}
		}
	}

     return agentExists;

   }

     /**
     Returns if there is an agent who has a corresponding
     Robot avatar at a specified gridpoint
     This method is used when the round robin scheme for handling
     agents is being used.
     @param Gridpoint
     @return true if such an agent is found else false
     */

   public synchronized boolean isRRAgentWithRAt(GridPoint gp)
   {

	Vector agentThreads = wwServer.getAgentList();

	if(!isValid(gp))
	 return false;

    boolean agentExists = false;



	if(agentThreads!=null)
	{
		for(int t = 0; t < agentThreads.size(); t++ )
		{
			WWServer.RoundRobinAgent x = (WWServer.RoundRobinAgent) agentThreads.elementAt(t);
			MovingObject mo = x.getCurrentLocation();

			if(mo!=null)
			{
				if(gp.getPtX() == mo.getLocation().getPtX() && gp.getPtY() == mo.getLocation().getPtY())
				{
					System.out.println(x.getAvataar());
					if(x.getAvataar()!=null)
						if(x.getAvataar().equals("Robot"))
						{
				 			agentExists = true;
						}
					break;
			 	}
			}
		}
	}
	else
	{

	}

     return agentExists;

   }

     /**
     Returns if there is an agent who has a corresponding
     physical avatar (robot or human) at a specified gridpoint
     This method is used when the round robin scheme for handling
     agents is being used.
     @param Gridpoint
     @return true if such an agent is found else false
     */

   public synchronized boolean isRRAgentWithR_HAt(GridPoint gp)
   {

	Vector agentThreads = wwServer.getAgentList();
	if(!isValid(gp))
	 return false;

    boolean agentExists = false;



	if(agentThreads!=null)
	{
		for(int t = 0; t < agentThreads.size(); t++ )
		{
			WWServer.RoundRobinAgent x = (WWServer.RoundRobinAgent) agentThreads.elementAt(t);
			MovingObject mo = x.getCurrentLocation();
			if(mo!=null)
			{
				if(gp.getPtX() == mo.getLocation().getPtX() && gp.getPtY() == mo.getLocation().getPtY())
				{
					if(x.getAvataar()!=null)
						if(x.getAvataar().equals("Robot") || x.getAvataar().equals("Human"))
						{
				 			agentExists = true;
						}
					break;
			 	}
			}
		}
	}

     return agentExists;

   }




   /**
   Returns whether there is an agent at a location in Wumpus World
   @param Gridpoint
   @return true if agent exists else false
   */


   public synchronized boolean isAgentAt(int x, int y)
   {

	Vector agentThreads = wwServer.getAgentThreads();
	if(!isValid(new GridPoint(x,y)))
	 return false;

    boolean agentExists = false;



	if(agentThreads!=null)
	{
		for(int t = 0; t < agentThreads.size(); t++ )
		{
			MovingObject mo = ((WWServer.AgentThread) agentThreads.elementAt(t)).getCurrentLocation();
			if(mo!=null)
			{
				if(x == mo.getLocation().getPtX() && y == mo.getLocation().getPtY())
				{
				 agentExists = true;
				 break;
			 	}
			}
		}
	}

     return agentExists;

   }
   /**
   Returns true if there is a wumpus at (x, y).
   @param location x and y
   @return TRUE if wumpus is at (x,y), FALSE otherwise.
   */
   public synchronized boolean isWumpusAt(int x, int y)
   {

	if(!isValid(new GridPoint(x, y)))
	 return false;

    boolean wumpusExists = false;
     Vector v = new Vector();
     v = getObjectAtLocation(x, y);

     if(!v.isEmpty())
     {
      Enumeration e;
      e = v.elements();

      while(e.hasMoreElements())
      {
       if(e.nextElement() instanceof Wumpus)
        wumpusExists = true;
      }
     }
    return wumpusExists;
   }


   /**
   Returns the direction or facing of the wumpus if there is a wumpus at (x, y).
   Returns -99 otherwise.
   @param location x and y
   @return facing of the wumpus or -99 if no wumpus.
   */
   public synchronized int getWumpusFacing(GridPoint gp)
   {
    int wumpusFacing = -99;
    int x = gp.getPtY();
    int y = gp.getPtX();
     Vector v = new Vector();
     v = getObjectAtLocation(x, y);


     for(int i = 0; i<v.size(); ++i)
     {
      if(v.elementAt(i) instanceof Wumpus)
      {
       Wumpus w = (Wumpus)v.elementAt(i);
       wumpusFacing = w.getOrientation();
      }
     }
     return wumpusFacing;
   }


   /**
   Returns true if there is a wumpus in the 4 squares surrounding the
   grid point.
   @param Gridpoint
   @return TRUE if wumpus is in surrounding locations, FALSE otherwise.
   */
   public synchronized boolean isWumpusNear( GridPoint gp)
   {
    boolean wumpusNear = false;
    int wx = gp.getPtX();
    int wy = gp.getPtY();
    if(isWumpusAt(wx-1, wy) ||(isWumpusAt(wx+1, wy))||(isWumpusAt(wx,wy-1)) || (isWumpusAt(wx,wy+1)))
      wumpusNear = true;
    return wumpusNear;
   }

   /**
   Returns an integer indicating the position of a wumpus at a
   certain gridpoint, if at all the location contains a wumpus.
   */
   public synchronized int getWumpusAt(GridPoint gp)
   {
    int wump = -1;

    if(!isValid(gp))
    {
      return wump;
    }

    Vector v = getObjectAtLocation(gp.getPtX(), gp.getPtY());
    int i;
    for(i = 0; i<v.size(); ++i)
    {
      if(v.elementAt(i) instanceof Wumpus)
        {
          wump = i;
          break;
        }
    }

    return wump;
   }


   /**
   Positions the wumpii to their next calculated location.
   Called at the end of each step in the simulator.
   */
   public  void processWumpi()
   {
    allWumpii.trimToSize();
    int s = allWumpii.size();

    for(int i = 0; i < s ; ++i)
    {
     Wumpus wumpusAtI = (Wumpus)allWumpii.elementAt(i);

     removeWumpus(wumpusAtI);
     wumpusAtI.findAction(this);

     wumpusAtI.act(true);
     addWumpus(wumpusAtI);
    }
   }

   /**
   Processes the shooting of a wumpus from a  location pt  by an agent.
   @param Agent location and orientation
   @return TRUE if wumpus is shot, FALSE otherwise.
   */
   public synchronized boolean shootAtWumpus(GridPoint pt, int orientation)
   {
    //int w=1;
    boolean flag = false;
    GridPoint gp = new GridPoint(pt.getPtX(), pt.getPtY());

    if(orientation == Directions.EAST)
    {
      while(isValid(gp))  //dont have to check at gp, since if wumpus is at gp, agent is Dead.
      {
        gp.advanceRight();
        if(isValid(gp))
        {
        	if(isWumpusAt(gp.getPtX(), gp.getPtY()))
        	{
				/* How should it work if there is a avataar is in the way?? */
				/* Assume that the arrow can go through the avatar. So donot bother
				   if there is anything in between
				*/
				flag = removeWumpus(gp);
				if(flag)
				 	break;
			}
		}
		else
			break;


      }
    }
    else if(orientation == Directions.WEST)
    {
      while(isValid(gp))
      {
        gp.advanceLeft();
        if(isValid(gp))
        {
			if(isWumpusAt(gp.getPtX(), gp.getPtY()))
				flag = removeWumpus(gp);
				if(flag)
				 	break;

		}
		else
			break;

      }
    }
    else if(orientation == Directions.SOUTH)
    {
      while(isValid(gp))
      {
        gp.advanceDown();
        if(isValid(gp))
        {
			if(isWumpusAt(gp.getPtX(), gp.getPtY()))
				flag = removeWumpus(gp);
				if(flag)
				 	break;

		}
		else
			break;

      }
    }
    else if(orientation == Directions.NORTH)
    {
      while(isValid(gp))
      {
        gp.advanceUp();
        if(isValid(gp))
        {
        	if(isWumpusAt(gp.getPtX(), gp.getPtY()))
        		flag = removeWumpus(gp);
				if(flag)
				 	break;

		}
		else
			break;
      }
    }

    return flag;
   }

   /**
   Removes wumpus from the given location.
   @param location and position
   @return TRUE if successful, FALSE otherwise.
   */
   private synchronized boolean  removeWumpus(GridPoint gp, int position)
   {
     boolean flag = false;
     if(position >= 0)
     {
      Vector v = getObjectAtLocation(gp.getPtX(), gp.getPtY());

      if(v.elementAt(position) instanceof Wumpus)
      {
        allWumpii.remove(v.elementAt(position));
        v.removeElementAt(position);
      }
      flag = true;
     }

     return flag;
   }
   /**
   Removes a wumpus from a grid point.
   @param Gridpoint
   @return true if wumpus is removed else false
   */

   private synchronized boolean  removeWumpus(GridPoint gp)
   {
     boolean flag = false;
     Vector v = getObjectAtLocation(gp.getPtX(), gp.getPtY());
	 Enumeration e = v.elements();

	 int i = 0;

	 while(e.hasMoreElements())
	 {
	  if(e.nextElement() instanceof Wumpus)
	  {
		  flag = true;
		  break;
	  }
	  else
	    i++;
	 }

      if(v.elementAt(i) instanceof Wumpus)
      {
        allWumpii.remove(v.elementAt(i));
        v.removeElementAt(i);
      }
     return flag;
   }

   /**
   Removes a specific wumpus object.
   */
   private synchronized void removeWumpus(Wumpus wObj)
   {
	 System.out.println(" " + wObj.getLocation().getPtX() + ", " + wObj.getLocation().getPtY() );
     Vector v = getObjectAtLocation(wObj.getLocation().getPtX(),wObj.getLocation().getPtY());

     v.remove(wObj);
   }

   /**
   Adds a specific wumpus object
   */
   private synchronized void addWumpus(Wumpus wObj)
   {
     Vector v = getObjectAtLocation(wObj.getLocation().getPtX(), wObj.getLocation().getPtY());

     v.addElement(wObj);
   }


   /**
   Returns true if there is a pit at location (x, y).
   @param location x and y
   @return TRUE if pit is found at (x,y), FALSE otherwise.
   */
   public synchronized boolean isPitAt(int x, int y)
   {
    boolean pitExists = false;

	if(!isValid(new GridPoint(x, y)))
	 return false;

     Vector v = new Vector();
     v = getObjectAtLocation(x, y);

     if(!v.isEmpty())
     {
      Enumeration e;
      e = v.elements();

      while(e.hasMoreElements())
      {
       if(e.nextElement() instanceof Pit)
        pitExists = true;
      }
     }

     return pitExists;
   }

   /**
   Returns true if there is a pit in the 4 squares surrounding the
   grid point.
   @param Location gridpoint
   @return TRUE if pit is in surrounding gridpoints, FALSE otherwise.
   */
   public synchronized boolean isPitNear(GridPoint gp)
   {
    boolean pitNear = false;
    int wx = gp.getPtX();
    int wy = gp.getPtY();

    if(isPitAt(wx-1, wy) ||(isPitAt(wx+1, wy))||(isPitAt(wx,wy-1)) || (isPitAt(wx,wy+1)))
      {
       pitNear = true;
      }
    return pitNear;
   }

   /**
   Returns whether there is a gold treasure in the adjacent cells
   @param Gridpoint
   @return true if a gold treasure exists else false
   */

   public synchronized boolean isGoldNear(GridPoint gp)
   {
    boolean goldNear = false;
    int wx = gp.getPtX();
    int wy = gp.getPtY();

    if(isGoldAt(wx-1, wy) ||(isGoldAt(wx+1, wy))||(isGoldAt(wx,wy-1)) || (isGoldAt(wx,wy+1)))
      {
       goldNear = true;
      }
    return goldNear;
   }
   /**
   Returns whether there is an obstacle in the adjacent cells
   @param Gridpoint
   @return true if obstacle exists else false
   */

   public synchronized boolean isObstacleNear(GridPoint gp)
   {
    boolean obNear = false;
    int wx = gp.getPtX();
    int wy = gp.getPtY();

    if(isObstacleAt(wx-1, wy) ||(isObstacleAt(wx+1, wy))||(isObstacleAt(wx,wy-1)) || (isObstacleAt(wx,wy+1)))
      {
       obNear = true;
      }
    return obNear;
   }


   /**
   Returns true if there is gold at the grid location gp.
   @param Gridpoint
   @return TRUE if the gold is present at the given location, FALSE otherwise.
   */
   public synchronized boolean isGoldAt(GridPoint gp)
   {
	if(!isValid(gp))
	 return false;

    boolean goldExists = false;
    int gx = gp.getPtX();
    int gy = gp.getPtY();



     Vector v = new Vector();
     v = getObjectAtLocation(gx , gy);
     if(!v.isEmpty())
     {
      Enumeration e;
      e = v.elements();

      while(e.hasMoreElements())
      {
       if(e.nextElement() instanceof Gold)
        {
          goldExists = true;
          break;
        }
      }
     }

     return goldExists;
   }

   /**
   Returns true if there is gold at the location (x,y)
   @param Location x and y coordinates.
   @return TRUE if the gold is present at the given location, FALSE otherwise.
   */
   public synchronized boolean isGoldAt(int x, int y)
   {
		if(!isValid(new GridPoint(x, y)))
		 return false;

	 boolean goldExists = false;

     Vector v = new Vector();
     v = getObjectAtLocation(x , y);
     if(!v.isEmpty())
     {
      Enumeration e;
      e = v.elements();

      while(e.hasMoreElements())
      {
       if(e.nextElement() instanceof Gold)
        {
          goldExists = true;
          break;
        }
      }
     }

     return goldExists;


   }

   /**
   Returns true if there is a obstacle at location (x, y).
   @param location x and y coordinates
   @return TRUE if the obstacle is at (x,y), FALSE otherwise.
   */
   public synchronized boolean isObstacleAt(int x, int y)
   {

    if(!cp.isCompletelyInside(new Cell(x*12, y*12)))
      return true;

    boolean ObstacleExists = false;


     Vector v = new Vector();
     v = getObjectAtLocation(x, y);

     if(!v.isEmpty())
     {
      Enumeration e;
      e = v.elements();

      while(e.hasMoreElements())
      {
       if(e.nextElement() instanceof Obstacle)
       {

        ObstacleExists = true;
	   }
      }
     }


	return ObstacleExists;
   }

   /**
   Removes gold pieces at location gp.
   @param Gridpoint
   */
   public synchronized void removeGoldAt(GridPoint gp)
   {
    int gx = gp.getPtX();
    int gy = gp.getPtY();

     Vector v = new Vector();
     v = getObjectAtLocation(gx , gy);

     if(!v.isEmpty())
     {
      Enumeration e;
      e = v.elements();
      int i = 0;
      while(e.hasMoreElements())
      {
	   Object o = e.nextElement();
       if(o instanceof Gold)
       {
		allGold.remove(o);
        v.removeElementAt(i);

        return;
       }
       ++i;
      }
     }

   }


	public synchronized void stopWumpiiThreads()
	{

		if(w != null)
			w.setEnd(true);
	}

	private WumpusThread w = null;

	public void startWumpusThread()
	{
		w = new WumpusThread();
		w.start();
	}
	/**
		This class is responsible for calculating new locations to which wumpii move to
	*/
	private class WumpusThread extends Thread
	{
		private boolean bEnd;
		public WumpusThread()
		{
			bEnd = false;
			setWumpusTargets();
			setDaemon(true);
		}
		public void setEnd(boolean x)
		{
			bEnd = x;
		}
		public void run()
		{
			while(!bEnd)
			{
				System.out.println("To recalculate the wumpus position");
				try
				{
					Thread.sleep(3000);	// Sleep time of wumpus Thread should be customized
										// This is set to 1 second at this point!
					processWumpi();
					cp.repaint();

				}
				catch(InterruptedException e)
				{

				}
			}
		}
	}


}
