/**
	This class is used to make the agents who have finished the current trial wait
	for other agents who are yet to finish their trials.
	It is also used to make those agents wait who have finished their first trial while
	the Wumpus World Server is still accepting connections.
	This is used only in the FCFS scheme.

	This is a lock that every agent obtains a lock on once the agent is done with the current
	trial which was not the last trial in game.
	Now the last agent to finish the game will notify all the other agents waiting on this.
	Thus the next trial resumes.
 * @author Ashwini Kuntamukkala
 * @version 	%I%, %G%
 * @since       JDK1.2

*/

	public class Monitor
	{
		private int count;
		private WWServer wwServer;

		/**
			Constructor takes a parameter to access the Wumpus World Server.
			@param WWServer

		*/
		public Monitor(WWServer wwS)
		{
			count = 0;
			wwServer = wwS;
		}

		/**
			Returns the number of agents waiting on the monitor
			@return number of agents waiting on the monitor
		*/
		public int getCount()
		{
			return count;
		}
		/**
			Notifies all the agents waiting on the monitor

		*/


		public synchronized void notifyAllOnMonitor()
		{
			notifyAll();
		}

		/**
		This is a very important method.
		Any agent who finishes a trial which is not the last trial invokes this method.
		If some other agents are still playing the game or if the Wumpus World Server is
		still waiting to accept connections, then the agent waits on the monitor.

		If the agent is the last one to attempt to lock, then that is the end to the trial
		and it notifies all the agents waiting on this lock so that the next trial can
		begin.
		It reads in the default state of wumpus world from a file "CurWW.txt"
		When the trial was started for the first time, the locations of all the entities in the
		Wumpus World were written to this file.
		*/
        	public synchronized boolean hit()
        	{
			if(wwServer.getAgentThreads()!=null)

			if(wwServer.getAgentsConnected()!=0)
			{
				// Check to see if the server has stopped accepting any more connections
				// so that that total number of agents in the game can be known
				if (((++count)% wwServer.getAgentsConnected())==0 && (wwServer.getServerAccCons() == false))
				{
					 wwServer.getMainPanel().readWorld("CurWW.txt");
					 count = 0;
					 notifyAll();
				}
				else
				{
					 try
					 {
						wait();
					 }
					 catch (InterruptedException e)
					 {
					 }
				}
			}
			else
			{

					notifyAll();
			}
     		return true; // keep playing.
      }

	/**
		Resets the count to 0
	*/

      public void initializeCount()
      {
		  count = 0;
      }
}
