import java.util.Random;
import java.util.Enumeration;
import java.util.Vector;
import java.util.*;
import java.io.*;
import java.awt.Toolkit;

/**
Specifies the object containing various percepts. The percepts
include extended percepts, natural language percepts and image percepts.
@author Kruti Mehta
 * @version 	%I%, %G%
 * @since       JDK1.2

*/
public class Percept
{
 /** Array containing image as pixels */
 private int[][][] pixels;
 private int imageHeight;
 private int imageWidth;
 private int totalImageWidth;
 private int totalImageHeight;

 private boolean stench;
 private boolean breeze;
 private boolean glitter;
 private boolean bump;
 private boolean scream;
 private boolean bumpedLastTime;
 private boolean wumpusDiedLastTime;

 private String hint;

 private Random randGen;

 private Environment env;
 /**
 Constructor. Initializes various percepts.
 */
 public Percept(int clientNo, Environment e)
 {
  Date d = new Date();
  randGen = new Random(d.getTime() * (clientNo + 1));
  stench = breeze = glitter = bump = scream = false;
  hint = new String("");
  pixels = new int[64][64][3];

  for(int i=0;i<64;++i)
   for(int j=0; j<64;++j)
    for(int k=0; k<3; ++k)
     pixels[i][j][k] = 0;

  env = e;
 }

 /**
 Used to set the percepts for each agent depending on their position
 and orientation in the grid. The Vector containing all the other agents
 is needed to calculate the natural language hints.

 public void perceive(GridPoint gp,  int orientation, Vector allAgents, TransferPercept tp)
 */
// public void perceive(GridPoint gp,  int orientation, wP.TransferPercept tp)
 public void perceive(GridPoint gp,  int orientation, ConInfo tp)
 {
  setStench(env, gp);
  setBreeze(env, gp);
  setGlitter(env,gp);
  setBump(env, gp);
  tp.setPercept(stench, breeze, glitter, bump);



  setHint(env,gp, orientation);


  setImage(env,gp,orientation);


 }


 /**
 Sets the stench percept for the agent.
 */
 private void setStench(Environment env, GridPoint gp)
 {
  stench = env.isWumpusNear(gp);
 }

 /**
 Sets the breeze percept for the agent.
 */
 private void setBreeze(Environment env, GridPoint gp)
 {
  breeze = env.isPitNear(gp);
 }

 /**
 Sets the image by reading it from the image file.
 */
 private void setImage(Environment env, GridPoint gp, int orientation) //,Vector allAgents)
 {
  int wumpusFacing = -99;
  boolean pitNext = false;
  boolean goldNext = false;
  boolean obstacleNext = false;
  int agentFacing = -99;
  boolean wumpusAt = false;
  boolean humanAt = false;
  boolean robotAt = false;
  for(int i=0;i<64;++i)
   for(int j=0; j<64;++j)
    for(int k=0; k<3; ++k)
     pixels[i][j][k] = 0;

  totalImageWidth = 0;
  totalImageHeight = 0;
  imageWidth = 0;
  imageHeight = 0;

  // Find if there is gold at this position
  GridPoint gpNext = new GridPoint(gp.getPtX(),gp.getPtY());

  // Move in the current direction the agent is facing
  if(orientation == Directions.SOUTH)
   gpNext.advanceDown();
  else if(orientation == Directions.WEST)
   gpNext.advanceLeft();
  else if(orientation == Directions.EAST)
   gpNext.advanceRight();
  else
   gpNext.advanceUp();



 // Determine if the new position is not valid or has an obstacle
  if(env.isValid(gpNext) == false || env.isObstacleAt(gpNext.getPtX(),gpNext.getPtY()) == true)
   obstacleNext = true;
  else
   {
	// If not, ie, the location is valid, then find if there is a pit at that location
    if(env.isPitAt(gpNext.getPtX(),gpNext.getPtY()) == true)
     pitNext = true;

    // If there was a wumpus at that location
	 wumpusAt = env.isWumpusAt(gpNext.getPtX(), gpNext.getPtY());

	 if(env.isGoldAt(gpNext)== true)
	     goldNext = true;


     humanAt = false;
     if(env.isAgentWithHAt(gpNext) || env.isRRAgentWithHAt(gpNext))
     	humanAt = true;

     robotAt = false;
     if(env.isAgentWithRAt(gpNext) || env.isRRAgentWithRAt(gpNext))
     	robotAt = true;

   }

  int imageX = 0;
  int imageY = 0;

  // Create gold image when gold is found at the current location
  if(goldNext == true)
   createGoldImage(imageX,imageY);

  System.out.println("\n************* imgX = " + imageX + " , imgY = " + imageY);

  // Create a pit image when a pit is found at the current location
  if(pitNext == true)
   //createPitImage(imageX,imageY);
   createPitImage(imageX, imageY);



  // Create image of a wumpus if a wumpus is found at this location
  //if(wumpusFacing != -99)
   //createWumpusImage(imageX,imageY,wumpusFacing);
  if(wumpusAt)
   createWumpusImage(imageX,imageY,3); //wumpusFacing);


  if(humanAt)
   createHumanImage(imageX,imageY); //wumpusFacing);

  if(robotAt)
   createRobotImage(imageX,imageY);


  // Create image of an obstacle when obstacle is found
  if(obstacleNext == true)
   createObstacleImage(imageX,imageY);


  imageWidth = 64;
  imageHeight = 64;
 }

/**
	This method creates a human image from a file called "human.ppm"
*/
 public void createHumanImage(int imageX, int imageY)
 {
	 try
	 {
	    FileInputStream fip = new FileInputStream("human.ppm");
    	readImage(fip,imageX+32,imageY+32);

	 }
	 catch(FileNotFoundException fne)
     {
      System.out.println("Percept: Image File" + " human.ppm " + " Not Found");
     }

 }

/**
	This method creates a robot image from a file called "robot.ppm"
*/

 public void createRobotImage(int imageX, int imageY)
 {
	 try
	 {
	    FileInputStream fip = new FileInputStream("robot.ppm");
    	readImage(fip,imageX+32,imageY+32);

	 }
	 catch(FileNotFoundException fne)
     {
      System.out.println("Percept: Image File" + " robot.ppm "+ " Not Found");
     }

 }

 /**
 	This method creates a wumpus image
 */

 public void createWumpusImage(int imageX, int imageY, int wumpusFacing)
 {
  String filename = new String();

  //Change the image files
  if(wumpusFacing == Directions.SOUTH)
  	//filename = "human.ppm";
   filename = "wumpusFront.ppm";
  else if(wumpusFacing == Directions.WEST)
   filename = "wumpusLeft.ppm";
  else if(wumpusFacing == Directions.EAST)
   filename = "wumpusRight.ppm";
  else if(wumpusFacing == Directions.NORTH)
   filename = "wumpusBack.ppm";

  //imageY = imageHeight;
  try
   {
    FileInputStream fip = new FileInputStream(filename);
    readImage(fip,imageX,imageY);
   }
   catch(FileNotFoundException fne)
   {
    System.out.println("Percept: Image File" + filename + " Not Found");
   }
 }

/**
	This method creates an image of an agent.
*/

 public void createAgentImage(int imageX, int imageY, int agentFacing)
 {
  String filename = new String();

  //Change the image files
  if(agentFacing == Directions.SOUTH)
   filename = "playerFront.ppm";
  else if(agentFacing == Directions.WEST)
   filename = "playerLeft.ppm";
  else if(agentFacing == Directions.EAST)
   filename = "playerRight.ppm";
  else if(agentFacing == Directions.NORTH)
   filename = "playerBack.ppm";

  imageY = imageHeight;
  imageX = imageWidth;
  try
   {
    FileInputStream fip = new FileInputStream(filename);
    readImage(fip,imageX,imageY);
   }
   catch(FileNotFoundException fne)
   {
    System.out.println("Percept: Image File" + filename + " Not Found");
   }
 }

/**
	This method creates an image of gold treasure
*/

 public void createGoldImage(int imageX, int imageY)
 {
  try
   {
    FileInputStream fip = new FileInputStream("gold.ppm");
    readImage(fip,imageX,imageY+32);
   }
   catch(FileNotFoundException fne)
   {
    System.out.println("Percept: Image File gold.ppm Not Found");
   }
 }

/**
	This method creates an image of a pit
	*/

 public void createPitImage(int imageX, int imageY)
 {
  try
   {
    FileInputStream fip = new FileInputStream("pit.ppm");
    readImage(fip,imageX+32,imageY);

   }
   catch(FileNotFoundException fne)
   {
    System.out.println("Percept: Image File pit.ppm Not Found");
   }
 }

/**
	This method creates an image of an obstacle
*/

 public void createObstacleImage(int imageX, int imageY)
 {
  try
   {
    FileInputStream fip = new FileInputStream("obstacle32.ppm");
    readImage(fip,imageX+32,imageY+32);
   }
   catch(FileNotFoundException fne)
   {
    System.out.println("Percept: Image File obstacle.ppm Not Found");
   }
 }
/**
Returns the array containing pixel values of the image being sent as a
percept to the client.
int[][][0] contains the red pixel values,
int[][][1] contains the green pixel values and
int[][][2] contains the blue pixel values.
*/
public int[][][] getImagePixels()
{
 return pixels;
}

/**
Returns the height of the image being sent to the agent.
*/
public int getImageHeight()
{
 return imageHeight;
}

/**
Returns the width of the image being sent to the agent.
*/
public int getImageWidth()
{
 return imageWidth;
}


  /**
  Reads the image from a file given that the file contains an
  image in ascii PPM format.
  */

 public void readImage (FileInputStream is, int imageX, int imageY)
 {
  DataInputStream dis = null;
  BufferedInputStream bis = null;
  System.out.println("ImageX "+ imageX +", ImageY " + imageY);
  try
  {
     bis = new BufferedInputStream (is);
     dis = new DataInputStream (bis);
     String word;
     word = readWord (dis);
     if (!"P3".equals (word))
     {
      System.out.println("INVALID IMAGE FORMAT");
     }

     imageWidth = Integer.parseInt (readWord (dis));
     imageHeight = Integer.parseInt (readWord (dis));

     int maxColors = Integer.parseInt (readWord (dis));
     if ((maxColors < 0) || (maxColors > 255))
     {
      System.out.println("INVALID IMAGE FORMAT");
     }

     for (int i=0;i<imageHeight;i++)
     {
      for (int j=0;j<imageWidth;j++)
      {
       int red = Integer.parseInt (readWord (dis));
       pixels[i+imageX][j+imageY][0] = red;
       int green = Integer.parseInt (readWord (dis));
       pixels[i+imageX][j+imageY][1] = green;
       int blue = Integer.parseInt (readWord (dis));
       pixels[i+imageX][j+imageY][2] = blue;
      }
     }
  }
  catch (IOException io)
  {
   System.out.println ("IO Exception " + io.getMessage());
  }
  catch (NoSuchElementException nse)
  {
   System.out.println ("No Such Element Exception " + nse.getMessage());
  }
  finally
  {
   try
   {
    if (dis != null)
       dis.close();
    if (bis != null)
       bis.close();
    if (is != null)
       is.close();
   }
   catch (IOException io)
   {
    System.out.println ("IO Exception " + io.getMessage());
   }
  }
 }

  /**
  Returns a word of text from stream. The word defined to be something
  wrapped by whitespace
  */
    private String readWord (InputStream is) throws IOException {
        StringBuffer buf = new StringBuffer();
        int b;
        do { // get rid of leading whitespace
            if ((b=is.read()) == -1)
                throw new EOFException();
            if ((char)b == '#') {  // read to end of line - ppm comment
                DataInputStream dis = new DataInputStream (is);
                dis.readLine();
                b = ' ';  // ensure more reading
            }
        } while (Character.isSpace ((char)b));
        do {
            buf.append ((char)(b));
            if ((b=is.read()) == -1)
                throw new EOFException();
        } while (!Character.isSpace ((char)b));  // reads first space
        return buf.toString();
    }

 /**
 Sets glitter percept.
 */
 private void setGlitter(Environment env, GridPoint gp)
 {
  glitter = env.isGoldAt(gp);  // was isGoldAt
 }

 /**
 Sets bump percept.
 */
 private void setBump(Environment env, GridPoint gp)
 {

   bump = env.isObstacleNear(gp);
 }

 /**
 Used to indicate that in the last step, the agent tried to walk into
 a wall or an obstacle.
 */
 public void setBumpedLastTime()
 {
  bumpedLastTime = true;
 }

 private void setScream(Environment env, GridPoint gp)
 {
  scream = false;
  if(wumpusDiedLastTime)
  {
    scream = true;
    wumpusDiedLastTime = false;
  }
 }

 /**
 Used to indicate that in the last step, the agent shot a wumpus.
 */
 public void setWumpusDiedLastTime()
 {
  wumpusDiedLastTime = true;
 }

 /**
 Returns true if the agent feels a breeze.
 */
 public boolean isBreeze()
 {
   return breeze;
 }

 /**
 Returns true if the agent smells the wumpus.
 */
 public boolean isStench()
 {
  return stench;
 }

 /**
 Returns true if the agent senses a glitter.
 */
 public boolean isGlitter()
 {
  return glitter;
 }

 /**
 Returns true if the agent bumped into a wall in the previous step.
 */
 public boolean isBump()
 {
  return bump;
 }

 /**
 Returns true if the agent hears a wumpus die.
 */
 public boolean isScream()
 {
  return scream;
 }


 public String getHint()
 {
  return hint;
 }

 /**
 Returns a truthful natural language hint for the agent.
 */
 private void setHint(Environment env, GridPoint gp, int orientation) //, Vector allAgents)
 {
  GridPoint pos = new GridPoint(gp.getPtX(), gp.getPtY());
  hint = "";
   setObjectHint(env,gp,orientation);

 }

 private void setWallHint(Environment env, GridPoint gp, int orientation)
 {
  int rand2 = randGen.nextInt(2);
  int rand3 = randGen.nextInt(2);
  GridPoint gpCopy = new GridPoint(gp.getPtX(),gp.getPtY());

  gpCopy.setPtX(rand2 * (env.getSize()-1));
  gpCopy.setPtY(rand3 * (env.getSize()-1));

  int rand4 = randGen.nextInt(3);

  switch(rand4)
  {
   case 0:
    hint = "there is a wall next to ";
    hint = hint + subjectPhrase(env,gpCopy);
    break;

   case 1:
    hint = "a wall is next to ";
    hint = hint + subjectPhrase(env,gpCopy);
    break;

   case 2:
    hint = hint + subjectPhrase(env,gpCopy);
    hint = hint + "is next to a Wall ";
    break;
  }


 }


 private void setObjectHint(Environment env, GridPoint gp, int orientation) //, Vector allAgents)
 {

  GridPoint selLoc = new GridPoint();

  int selObjectType = objectSel(env,selLoc);
  if(selObjectType==-1)
   return;

  int rand4 = randGen.nextInt(3);
  switch(rand4)
  {
   case 0:
    hint = hint + subjectPhrase(env,selLoc,selObjectType);
    System.out.println(hint);
    hint = hint + "is ";
    hint = hint + propositionalPhrase(selLoc,gp,orientation);
   break;

   case 1:
    hint = hint + "there is ";
    hint = hint + subjectPhrase(env,selLoc,selObjectType);
    hint = hint + propositionalPhrase(selLoc,gp,orientation);
   break;

   case 2:
    String temphint = predicatePhrase(env,selLoc,selObjectType);
    hint = hint + subjectPhrase(env,selLoc) + " ";
    hint = hint + temphint;
    break;
  }



 }

 private int objectSel(Environment env, GridPoint selLoc) //, Vector allAgents)
 {
  Vector v = new Vector();
  int totalObj = 0;
  int totalAgents = 0;
  Object selObject = new Object();
  int selObjectType = -1;
  Vector grid = new Vector();

  synchronized(env)
  {
 	  // Select all the grid locations which have atleast one entity
	  for(int i = 0; i < env.getWidth(); ++i)
	   for(int j = 0; j < env.getLength(); ++j)
	   {
			if(env.getObjectAtLocation(i,j).size() > 0)
			{
				grid.add(new GridPoint(i,j));
				System.out.println("" +i+","+j);
			}

	   }

	  // If there are no wumpii, pits, obstacles in the wumpus world
	  // then no grid locations will be selected
	  if(grid.size()==0)
	  {
		  hint = "There are no obstacles, wumpii or pits";
		  return selObjectType;
	  }

	  // Generates a random number in the wumpus world
	  System.out.println("The total number of objects are "+grid.size());

	  // Chooses any of the grid locations randomly
	  Vector v1 = null;
	  do
	  {
		  int rand1 = randGen.nextInt(grid.size());// + totalAgents);

		  GridPoint x = (GridPoint) grid.elementAt(rand1);
		  selLoc.setPtX(x.getPtX());
		  selLoc.setPtY(x.getPtY());
		  // Get the vector of objects in that location
		  v1 = env.getObjectAtLocation(selLoc.getPtX(), selLoc.getPtY());
		  System.out.println("This is in the loop");
	  }
	  while(v1==null);

	  int rand2 = randGen.nextInt(v1.size());
	  selObject = v1.elementAt(rand2);

	  if(selObject instanceof Wumpus)
	   selObjectType = GridObjectType.WUMPUS;
	  else if(selObject instanceof Gold)
	   selObjectType = GridObjectType.GOLD;
	  else if(selObject instanceof Pit)
	   selObjectType = GridObjectType.PIT;
	  else if(selObject instanceof Obstacle)
	   selObjectType = GridObjectType.OBSTACLE;



  }


   return selObjectType;
 }


 private String subjectPhrase(Environment env, GridPoint gp)
 {
  String subText;
  int rand1 = randGen.nextInt(2);

  if(rand1 == 0)
   subText = "the location ";
  else
   subText = "the square ";

  subText = subText + gp.toString();

  return subText;
 }

 private String subjectPhrase(Environment env, GridPoint selLoc, int selObjectType)
 {
  System.out.println("In Subject Phrase function....");
  String subText = "";
  int rand1 = randGen.nextInt(2);

  switch(selObjectType)
  {
   case GridObjectType.WUMPUS:
    if(rand1 == 0)
     subText = "a wumpus ";
    else
    {
     subText = "a stench ";
    }
   break;

   case GridObjectType.PIT:
    if(rand1 == 0)
     subText = "a pit ";
    else
    {
     subText = "a breeze ";
    }
   break;

   case GridObjectType.GOLD:
    if(rand1 == 0)
     subText = "a gold ";
    else
     subText = "a glitter ";
   break;

   case GridObjectType.OBSTACLE:
    subText = "an obstacle ";
   break;

   case GridObjectType.AGENT:
    subText = "an agent ";
   break;
  }

  return subText;
 }

 private void selPerceptLoc(Environment env, GridPoint selLoc)
 {
  int orientation = -1;
  GridPoint nextLoc = new GridPoint(selLoc.getPtX(), selLoc.getPtY());
  do
  {
   nextLoc.setPtX(selLoc.getPtX());
   nextLoc.setPtY(selLoc.getPtY());
   orientation = randGen.nextInt(4);
   if(orientation == Directions.EAST)
    nextLoc.setPtX(selLoc.getPtX()+1);
   else if(orientation == Directions.WEST)
    nextLoc.setPtX(selLoc.getPtX()-1);
   else if(orientation == Directions.NORTH)
    nextLoc.setPtY(selLoc.getPtY()+1);
   else
    nextLoc.setPtY(selLoc.getPtY()-1);
  } while((nextLoc.getPtX() < 0) || (nextLoc.getPtX() >= env.getSize()) ||
          (nextLoc.getPtY() < 0) || (nextLoc.getPtY() >= env.getSize()) );

  selLoc.setPtX(nextLoc.getPtX());
  selLoc.setPtY(nextLoc.getPtY());
 }

 private String propositionalPhrase(GridPoint selLoc, GridPoint gp, int orientation)
 {

  boolean adjacent = false;
  int dir = -1;
  String subText = "";

  if(selLoc.getPtX() == gp.getPtX() + 1 && selLoc.getPtY() == gp.getPtY())
  {
   adjacent = true;
   dir = Directions.EAST;  //2
  }
  else if(selLoc.getPtX() == gp.getPtX() - 1 && selLoc.getPtY() == gp.getPtY())
  {
   adjacent = true;
   dir = Directions.WEST;  //4
  }
  if(selLoc.getPtX() == gp.getPtX() && selLoc.getPtY() == gp.getPtY() + 1)
  {
   adjacent = true;
   dir = Directions.SOUTH; //1
  }
  if(selLoc.getPtX() == gp.getPtX() && selLoc.getPtY() == gp.getPtY() - 1)
  {
   adjacent = true;
   dir = Directions.NORTH; //3
  }

  if(adjacent)
  {
   int offset = dir - orientation - 1;
   if(offset < 0)
    offset = offset + 4;
   switch(offset)
   {

    case 0:
     subText = "to your left ";
    break;

    case 1:
     subText = "behind you ";
    break;

    case 2:
     subText = "to your right ";
    break;

    case 3:
     subText = "ahead of you ";
    break;


   }
  }

  else
  {
   if(randGen.nextInt(2) == 0)
    subText = "in ";
   else
    subText = "at ";

   if(randGen.nextInt(2) == 0)
    subText = subText + "location ";
   else
    subText = subText + "square ";

   subText = subText + selLoc.toString();
  }

  return subText;
 }

 private String predicatePhrase(Environment env, GridPoint selLoc, int selObjectType)
 {
  GridPoint orig = new GridPoint(selLoc.getPtX(),selLoc.getPtY());
  String subText = "";
  String objText = "";

  objText = subjectPhrase(env,selLoc,selObjectType);

  switch(selObjectType)
  {
   case GridObjectType.WUMPUS:
    if(orig.getPtX() == selLoc.getPtX() && orig.getPtY() == selLoc.getPtY())
    {
     if(randGen.nextInt(2) == 0)
      subText = "contains ";
     else
      subText = "has ";
     subText = subText + objText;
    }
    else
    {
     switch(randGen.nextInt(4))
     {
      case 0:
       subText = "is smelly ";
       break;
      case 1:
       subText = "smells bad ";
       break;
      case 2:
       subText = "has " + objText;
       break;
      case 3:
       subText = "contains " + objText;
       break;
     }
    }
    break;

   case GridObjectType.PIT:
    if(orig.getPtX() == selLoc.getPtX() && orig.getPtY() == selLoc.getPtY())
    {
     if(randGen.nextInt(2) == 0)
      subText = "contains ";
     else
      subText = "has ";
     subText = subText + objText;
    }
    else
    {
     switch(randGen.nextInt(4))
     {
      case 0:
       subText = "is breezy ";
       break;
      case 1:
       subText = "feels breezy ";
       break;
      case 2:
       subText = "has " + objText;
       break;
      case 3:
       subText = "contains " + objText;
       break;
     }
    }
    break;

   case GridObjectType.GOLD:
    switch(randGen.nextInt(3))
    {
     case 0:
      subText = "looks glittery ";
      break;
     case 1:
      subText = "has " + objText;
      break;
     case 2:
      subText = "contains " + objText;
      break;
     }
     break;

   case GridObjectType.OBSTACLE:
    switch(randGen.nextInt(3))
    {
     case 0:
      subText = "looks bumpy ";
      break;
     case 1:
      subText = "has " + objText;
      break;
     case 2:
      subText = "contains " + objText;
      break;
     }
     break;

    case GridObjectType.AGENT:
     switch(randGen.nextInt(2))
     {
      case 0:
       subText = "has " + objText;
       break;
      case 1:
       subText = "contains " + objText;
       break;
      }
     break;

   }
  return subText;
 }
}
