
/**
Keeps track of the score statistics for each agent.
  @author Kruti Mehta
 * @version 	%I%, %G%
 * @since       JDK1.2

*/
public class ScoreKeeper
{
  private int noOfTrials;
  private int maxTrials;
  int[] gameScore;
  double averageScore;
  double stdDeviation;
  int    agentID;
  /**
  Constructor. Initializes score information.
  */
  public ScoreKeeper(int numTrials)
  {
	// Sets the maximum number of trials per game
    maxTrials = numTrials;
    gameScore = new int[maxTrials + 1];
    for(int i = 1; i <= maxTrials; ++i)
      gameScore[i] = 0;

    noOfTrials = 1;

  }

  public void setAgentID(int a)
  {
	  agentID = a;
  }
 /*
   Used to set the current score of the agent.
 */
  public void setScoreForTrial(int score, int trialNumber)
  {

	  if(trialNumber <= maxTrials)
	  {
		noOfTrials = trialNumber;
	  	gameScore[trialNumber] = score;
	  }
  }

  /**
  Calculates average score for the completed trials.
  */
  public void calculateAvgScore()
  {
    int total = 0;
    for(int i = 1; i <= noOfTrials; ++i)
    {
      System.out.println("gameScore for trial " + i + " is " + gameScore[i]);
      total += gameScore[i];
    }
    averageScore = (double) ((total*1.0)/(noOfTrials*1.0));
  }

  /**
  Calculates scores standard deviation for the completed trials.
  */

  public void calculateStdDeviation()
  {
   double sum = 0;
   for(int i = 1; i <=noOfTrials; ++i)
    sum = sum + (gameScore[i]-averageScore) * (gameScore[i]-averageScore);
   sum = sum / noOfTrials;
   stdDeviation = Math.sqrt(sum);
  }

  public double getAverageScore()
  {
   return averageScore;
  }

  public double getStdDeviation()
  {
   return stdDeviation;
  }

  public String toString()
  {
	  String s = "\nAgent ID = " + agentID;
	  for(int i = 1; i <= noOfTrials; i++)
	  {
		s +=  "\nTrial["+i+"] = " + gameScore[i];
	  }
	  calculateAvgScore();
	  calculateStdDeviation();
	  s += "\nAverage Score = " + averageScore +"\nStandard Deviation = " + stdDeviation;

	  return s;
  }

}