import java.awt.*;
import java.awt.geom.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.image.*;
import java.util.*;
import java.io.*;
import java.net.*;

public class WWServer extends JFrame
{
	private RobotDetails rbDetails;
	private HumanAvatarDetails hmnDetails;
	private JMenuBar    bar;
	private JMenu		fileItem;
	private JMenu		toolMenu;
	private JMenuItem	openItem;
	private JMenuItem	saveItem;
	private JMenuItem	exitItem;
	private JMenu		helpMenu;
	private JMenuItem   aboutItem;
	private JMenuItem   optionsItem;

	private JMenu 		simulatorMenu;
	private JMenuItem	configureItem;
	private JMenuItem   startItem;
	private JMenuItem   printItem;
	private JMenuItem   robotItem;
	private JMenuItem   humanItem;

	private String 		wumpusMode;
	private JCheckBoxMenuItem	toolboxItem;
	private MainContentPanel mainPanel;
	private WumpusModeDialog dialog;
	private ServerSocket server;
	Socket connection;
	private CustomizedPanel drawingArea;
 	private int noofTrials;
 	private int maxSteps;
 	private int noofClients;
 	private int clientID[];
 	private int portNumber;
	private boolean b_fcfs;
	private boolean b_roundRobin;
	private int currentClientNumber;
	private String  sel_wumpusMode;
	private Vector  agentThreads;
	private Vector  scoreKeeper;

	private int nRobotsAvail;
	private int nHumansAvail;
	private int timeToAcceptCon;
	private int timeToAcceptActions;

	private boolean serverAccCons;
	private boolean firstTimeStart;
	private ScoreBoardForAgents sbfa = null;
	private Vector agentList = null;

	public WWServer(String title)
	{
		setTitle(title);
		MenuHandler mHandler = new MenuHandler();
		bar = new JMenuBar();
		setJMenuBar(bar);
				fileItem = new JMenu("File");
				fileItem.setMnemonic('F');
				fileItem.addMouseListener(
					new MouseAdapter()
					{
						public void mouseClicked(MouseEvent e)
						{
						}
					}
					);
				fileItem.setVisible(true);
				openItem  = new JMenuItem("Load Wumpus World");
				openItem.setMnemonic('L');
				openItem.addActionListener(mHandler);
				openItem.setVisible(true);
				fileItem.add(openItem);

				saveItem  = new JMenuItem("Save Wumpus World");
				saveItem.setMnemonic('S');
				saveItem.addActionListener(mHandler);
				fileItem.add(saveItem);

				fileItem.addSeparator();

				exitItem = new JMenuItem("Exit");
				exitItem.addActionListener(
					new ActionListener()
					{
						public void actionPerformed(ActionEvent e)
						{
							closeWWServer();
							System.exit(0);
						}
					}
					);

				addWindowListener(
					new WindowAdapter()
					{
					  public void windowClosing(WindowEvent e)
					  {
					  	closeWWServer();
					  	System.exit(0);
					  }
					}
				);
				exitItem.setMnemonic('x');
				fileItem.add(exitItem);


				bar.add(fileItem);

				toolMenu = new JMenu("Tools");
				toolMenu.setMnemonic('T');
				toolboxItem = new JCheckBoxMenuItem("Show ToolBar");
				toolboxItem.setSelected(true);
				toolboxItem.addItemListener(
					new ItemListener()
					{
						public void itemStateChanged(ItemEvent e)
						{
							if(toolboxItem.isSelected())
								{
									mainPanel.showToolbar(true);
								}
							else
								{
									mainPanel.showToolbar(false);

								}

						}
					}
					);
				optionsItem = new JMenuItem("Options");
				optionsItem.setMnemonic('O');
				toolMenu.add(optionsItem);
			    toolMenu.addSeparator();

			    robotItem = new JMenuItem("Robot Avatar Details");
			    robotItem.setMnemonic('R');

				humanItem = new JMenuItem("Human Avatar Details");
				humanItem.setMnemonic('H');

			    toolMenu.add(robotItem);
			    toolMenu.add(humanItem);


				rbDetails = new RobotDetails(new JFrame(), true);
			    robotItem.addActionListener(
					new ActionListener()
					{
						public void actionPerformed(ActionEvent e)
						{

							 rbDetails.setVisible(true);


						}
					}
					);

				hmnDetails = new HumanAvatarDetails(new JFrame(), true);
			    humanItem.addActionListener(
					new ActionListener()
					{
						public void actionPerformed(ActionEvent e)
						{

							 hmnDetails.setVisible(true);


						}
					}
					);




				toolMenu.addSeparator();

				toolMenu.add(toolboxItem);

				simulatorMenu = new JMenu("Simulator");
				simulatorMenu.setMnemonic('i');
				configureItem = new JMenuItem("Set Configuration");
				configureItem.setMnemonic('e');

				Handler h = new Handler();
				configureItem.addActionListener(h);
				simulatorMenu.add(configureItem);

				startItem = new JMenuItem("Status");
				startItem.setMnemonic('a');
				startItem.addActionListener(h);

				simulatorMenu.add(startItem);
				bar.add(simulatorMenu);


				helpMenu = new JMenu("Help");
				helpMenu.setMnemonic('H');
				aboutItem = new JMenuItem("About");
				helpMenu.add(aboutItem);
				aboutItem.addActionListener(
					new ActionListener()
					{
						public void actionPerformed(ActionEvent e)
						{
							new HelpDialog(new JFrame(), true).show();
						}
					}
					);


		setConfiguration();
		dialog = new WumpusModeDialog(new JFrame(), true);
		gsd    = new GameStatusDialog(new JFrame(), false);
		statusArea = gsd.getStatusArea();

		bar.add(toolMenu);
		bar.add(helpMenu);

		mainPanel = new MainContentPanel();
		drawingArea = mainPanel.getDrawingArea();
		drawingArea.setWWServer(this);

        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(mainPanel, BorderLayout.CENTER);
        panel = new JPanel(new FlowLayout());
        btnStart = new JButton("Start");
        btnStart.addActionListener(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					setGameStartMenu();
					// Saving the wumpus world details into the file
					// so that it can be recreated in the next trial
					if(firstTimeStart)
					{
						mainPanel.writeWorld("CurWW.txt");
						firstTimeStart = false;
					}
					else
						drawingArea.readWorld("CurWW.txt");

					startGame();
					repaint();
				}
			}
			);

		btnStatistics = new JButton("Statistics");
		btnStatistics.setEnabled(false);

        btnStatistics.addActionListener(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{

					if(sbfa!=null)
					{
						/*Find a score*/
						sbfa.showScoresOfAgents();
						sbfa.setSize(500,500);
						sbfa.show();
					}

				}
			}
			);

		btnClose = new JButton("End Game");
		btnClose.setEnabled(false);
		btnClose.addActionListener(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					JOptionPane.showMessageDialog(null, "Not implemented yet!!");
				}
			}
			);

        panel.add(btnStart);
        panel.add(btnStatistics);
		//panel.add(btnClose);
		wumpusMode = new String("Random");

        getContentPane().add(panel, BorderLayout.NORTH);




		drawingArea.getEnvironment().setWWServer(this);
        setSize(600,600);
        pack();
        setVisible(true);
		setSize(500,500);
		show();
	}

    public Vector getAgentThreads()
    {
		return agentThreads;
	}

	public Vector getAgentList()
	{
		return agentList;
	}
    public boolean isSchemeRR()
    {
		return b_roundRobin;
	}

	public boolean getServerAccCons()
	{
		return serverAccCons;
	}

	public MainContentPanel getMainPanel()
	{
		return mainPanel;
	}
    public synchronized void decrementNClients()
    {
		nClients--;
	}
    public synchronized int  getClientNumber()
    {
		int x = -1;	// => required number of agent have already connected to
					//    the ww server and no more connections can be accepted
		for(int i=0;i < noofClients; i++)
		{
			if(clientID[i]==0)
			{
				clientID[i] = (i+1);
				x = i;
				break;
			}

		}

		return x;

	}

	public synchronized int getAgentsConnected()
	{
		int x = 0;
		for(int i = 0; i < noofClients; i++)
		{
			// If an agent has connected to the web server.
			if(clientID[i] != 0)
			{
				x++;
			}
		}

		return x;
	}
	public void disconnectClient(int i)
	{
		clientID[i] = 0;
	}
	public void closeWWServer()
	{
		System.out.println("Closing wumpus world server");
		if(agentThreads!=null)
		{
			for(int i = 0; i < agentThreads.size(); i++)
			{
				AgentThread a = (AgentThread) agentThreads.elementAt(i);

				a.setTerminate();


			}
			agentThreads.removeAllElements();
			agentThreads = null;
		}

		drawingArea.getEnvironment().stopWumpiiThreads();

		try
		{
			Thread.sleep(5000);
		}
		catch(InterruptedException ie)
		{
			System.out.println ( "zhaoxiaoru6 " + ie.getMessage () );
			ie.printStackTrace ();
		}

	}
	public void setConfiguration()
	{
		 	noofTrials = 1;
		  	maxSteps = 1000;
		 	noofClients = 1; // Default changed to 1 by Dr. Y
		 	clientID = new int[noofClients];
		 	portNumber = 5000;
		 	currentClientNumber = 0;
			b_fcfs = true;
			b_roundRobin = false;

			timeToAcceptCon = 60*1000;

			timeToAcceptActions = 1000*120;

			firstTimeStart = true;
			nRobotsAvail = 0;
			nHumansAvail = 0;

	}
	public void setGameStartMenu()
	{
		serverAccCons = true;
		btnStart.setEnabled(false);
		btnStatistics.setEnabled(true);
		btnClose.setEnabled(true);
		openItem.setEnabled(false);
		saveItem.setEnabled(false);
		configureItem.setText("View Configuration");
		mainPanel.setButtonsEnabled(false);
		agentThreads = new Vector();
		scoreKeeper  = new Vector();

		sbfa = new ScoreBoardForAgents(scoreKeeper);
		sbfa.clearScoreBoard();
		clientID = new int[noofClients];
		for(int i = 0; i < noofClients; i++)
		{

			clientID[i] = 0;
		}

		nRobotsAvail = 0;
		nHumansAvail = 0;

	}

	public String getSelWumpusMode()
	{
		return sel_wumpusMode;
	}
	public void setGameEndMenu()
	{
		btnStart.setEnabled(true);

		btnClose.setEnabled(false);
		openItem.setEnabled(true);
		saveItem.setEnabled(true);
		configureItem.setText("Set Configuration");
		mainPanel.setButtonsEnabled(true);
		firstTimeStart = true;
		agentThreads = new Vector();
		clientID = new int[noofClients];
		for(int i = 0; i < noofClients; i++)
		{

			clientID[i] = 0;
		}

	}
	private JPanel panel;
	private JButton btnStart;
	private JButton btnStatistics;
	private JButton btnClose;
	private GameStatusDialog gsd;
	public void getWumpusMode()
	{

			dialog = new WumpusModeDialog(new JFrame(), true);
			dialog.setSize(200,175);
			dialog.setDimension(new Dimension(200,180));
			dialog.pack();
			dialog.setVisible(true);
			sel_wumpusMode = dialog.getMode();

	}

    private class Handler implements ActionListener
    {
		public void actionPerformed(ActionEvent e)
		{
			if(e.getSource()==configureItem)
			{
				if(configureItem.getText().equals("Set Configuration"))
				{
					ConfigureDialog c = new ConfigureDialog(new JFrame(), true);

					c.setSize(370,300);
					c.show();
					return;
				}
				else
				{
					ConfigureDialog c = new ConfigureDialog(new JFrame(), true);
					c.setOKDisabled();
					c.setSize(370,300);
					c.show();
				}
			}
			if(e.getSource()==startItem)
			{
				gsd.setVisible(true);


			}

		}
	}

	private JTextArea statusArea;
	private int nClients;

	public void startGame()
	{

		        ProgressBar mainFrame = new ProgressBar(new JFrame(), true);
				mainFrame.setSize(315, 135);
				mainFrame.setVisible( true );
				mainFrame.pack();

				MainThread mainThread = new MainThread(this);
				mainThread.start();
				// Launches the wumpus thread!!!!!
				drawingArea.setDefaultCursor();
				drawingArea.getEnvironment().startWumpusThread();

	}

	private class MainThread extends Thread
	{
		private Monitor monitor;
		public MainThread(WWServer wwS)
		{

			// Acts as a monitor for all the agents that will be
			// playing the game. It will be used to lock the
			// agents till all the agents finish the corresponding trial
			//
			monitor = new Monitor(wwS);
		}

		public void run()
		{
				  nClients = 0;
				  try
					{
						server = new ServerSocket(portNumber, 100);
						server.setSoTimeout(timeToAcceptCon);

						if(b_roundRobin)
						{

							statusArea.append("\n\tRound Robin Message Handling");
							statusArea.append("\n\tWaiting to accept requests for connections");
							agentList = new Vector();
							while(true)
							{
								connection = server.accept();
								nClients = getAgentsConnected();

								if(nClients != noofClients)
								{
									/*
										Check it the connection can be granted
										If yes, add the agentobject to the vector
									*/
									RoundRobinAgent rra = new RoundRobinAgent(connection);
									if(rra.wasConnectionAccepted())
										agentList.add(rra);

								}
							}
							/*
								Round Robin
							*/
						}
						else
						{

							/*
								First come First Serve
							*/
							serverAccCons = true;
							statusArea.append("\n         *************************************************************\n");
							while(true)
							{
								statusArea.append("\n\tWaiting for connections from "+Integer.toString(noofClients - nClients) + " Clients ....\n");
								setTitle("Wumpus World Server IP: "+ InetAddress.getLocalHost().getHostAddress()+", Port: "+ portNumber);
								connection = server.accept();

								// Obtain the existing number of agents connected to the
								// wumpus world server.
								nClients = getAgentsConnected();
								if(nClients!=noofClients)
								{

								statusArea.append("\n\tConnection : " + nClients + " received from " + connection.getInetAddress().getHostName());
								statusArea.append("\n\tNoof connections obtained = " + Integer.toString(nClients));



									new AgentThread(connection,false, monitor);
								}
								else
								{
									// This just starts the game now.
									new AgentThread(connection, true, monitor);
									statusArea.append("\n\n         *************************************************************\n");
									statusArea.append("\n\tStarting game ....");

									break;
								}

							}
						}

					}
					catch(InterruptedIOException e)
					{

						// Check if any connections from agents have been accepted.
						// If no connections have been obtained, then restart the wumpus world
						// server

						if(getAgentsConnected()==0)
						{
							JOptionPane.showMessageDialog(null, "No connections obtained from the agents. Please restart the wumpus world simulator");
							setGameEndMenu();

						}
						else
						{
							// Atleast one connection has been established => we need to
							// append this in the status area
							// Check if there are all the agents are waiting for the
							// next trial to start while the server was busy waiting
							// for accepting connections

							statusArea.append("\n\n         *************************************************************\n");
							statusArea.append("\n\tStarting game ....");

							if(b_roundRobin)
							{
								/*
									Here we have to send percepts Then collect actions Execute the action
								*/
								if(agentList!=null)
								{
									if(agentList.size()>0)
									{
										JOptionPane.showMessageDialog(null, "you are by buddy!");
										serveAgentsInRR();
									}
									else
										JOptionPane.showMessageDialog(null, "you are not by buddy!");

								}

							}
							else
							{
								System.out.println("The server is not accepting connections now");
							    if(monitor.getCount()==getAgentsConnected())
								{
									System.out.println("Notifying the agents");
									mainPanel.readWorld("CurWW.txt");
									monitor.initializeCount();
 									monitor.notifyAllOnMonitor();
								}
							}



						}
					}
					catch(IOException eof)
					{

						System.out.println ( "zhaoxiaoru6 " + eof.getMessage () );
						eof.printStackTrace ();
					}
					catch(Exception e)
					{

						System.out.println ( "zhaoxiaoru6 " + e.getMessage () );
						e.printStackTrace ();
					}

					try
					{
						// Signifies that the server is no longer accepting connections
						serverAccCons = false;

						// Closing the actual server socket that was accepting connections
						server.close();
					}
					catch(IOException io)
					{
						System.out.println ( "zhaoxiaoru6 " + io.getMessage () );
						io.printStackTrace ();
					}


		}


	}

	private void serveAgentsInRR()
	{
		try
		{


			int currentTrial = 1;
			boolean over = true;
			while(agentList.size()>0)
			{
				for(int i=0;i<agentList.size();i++)
				{
					RoundRobinAgent rra = (RoundRobinAgent) agentList.elementAt(i);
					if(currentTrial == rra.getCurrentTrial())//!rra.isTrialOver())
					{
						if(rra.sendPercept())
							if(rra.receiveAction())
								rra.executeAction();
					}
				}

				over = true;
				for(int j=0;j<agentList.size();j++)
				{
					RoundRobinAgent rra = (RoundRobinAgent) agentList.elementAt(j);
					if(rra.getCurrentTrial()==currentTrial)
					{
						over = false;
						break;
					}
				}

				if(over)
					currentTrial++;

				if(currentTrial > noofTrials)
				{
					JOptionPane.showMessageDialog(null, "Game over, Start a new one!");
					setGameEndMenu();
					break;
				}
			}
			JOptionPane.showMessageDialog(null, "No agents are currently playing the game... Game over, Start a new one!");
			setGameEndMenu();
		}
		catch(Exception e)
		{
			System.out.println("Some exception occurred");
		}
	}
	private class MenuHandler implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{

			if(e.getSource()==openItem)
			{
				JFileChooser f = new JFileChooser();
				f.setFileSelectionMode(JFileChooser.FILES_ONLY);
				int result = f.showOpenDialog(null);

				if(result==JFileChooser.APPROVE_OPTION)
				{
					File fileName = f.getSelectedFile();
					mainPanel.readWorld(fileName.getPath());
				}

			}
			else
			{

				if(e.getSource()==saveItem)
				{
					JFileChooser f = new JFileChooser();
					f.setFileSelectionMode(JFileChooser.FILES_ONLY);
					int result = f.showSaveDialog(null);

					if(result==JFileChooser.APPROVE_OPTION)
					{
						File filename = f.getSelectedFile();

						mainPanel.writeWorld(filename.getPath());
					}
				}
			}
		}
	}

	public synchronized void removeAgent(AgentThread a, int agentID)
	{
		boolean found = true;
		if(agentThreads!=null)
		{
			found = agentThreads.remove(a);
		}

		// This implies that all the agents have been disconnected or
		// have finished their game
		// So the game is done and restart the wumpus world
		if(found && agentThreads != null)
			if(agentThreads.size()==0 && serverAccCons==false)
			{
				JOptionPane.showMessageDialog(null, "The game is over. Please restart the wumpus world server");

				// Set the wumpus world simulator for restarting
				setGameEndMenu();
			}
		disconnectClient(agentID);
		repaint();
	}

	public static void main(String args[])
	{
				final WWServer app = new WWServer("Wumpus World Server");
				app.addWindowListener(
					new WindowAdapter()
					{
					  public void windowClosing(WindowEvent e)
					  {
						app.closeWWServer();
					  	System.exit(0);
					  }
					}
				);

	}



    class RoundRobinAgent
    {
		boolean conn =  false;
		private Socket socket;
		private ObjectInputStream input;
		private ObjectOutputStream output;
		private int clientNumber;
		private GridPoint gp;
		private boolean suffRsc;
		private String avataar;
		private String avataarIP;
		private int    avataarPort;
		private GridPoint startingPos;
		private GridPoint currentPos;
		private MovingObject mo;
		private int orgOrientation;
		private int orientation;
		private ScoreKeeper scoreBoard;
		private boolean wumpusShot;
		private boolean trialOver;
		private int count;

		private boolean bGame;
		private int trialScore;
		private int nArrows;
		private int nGoldPieces;
		private int actionTaken;

		private int currentStep;
		private int currentTrial;
		private DatagramSocket dgsock=null;
		private String s1; // Represents action sent by the agent.

		public RoundRobinAgent(Socket s)
		{
			try
			{
				socket = s;
				socket.setSoTimeout(timeToAcceptActions);
				count = 0;
				input = new ObjectInputStream(socket.getInputStream());
				output = new ObjectOutputStream(socket.getOutputStream());
				conn = connectionValid();
			}
			catch(EOFException eof)
			{
				conn = false;
			}
			catch(IOException ioex)
			{
				conn = false;
			}

		}

		public int getCurrentTrial()
		{
			return currentTrial;
		}
		public boolean wasConnectionAccepted()
		{
			return conn;
		}
		private boolean connectionValid()
		{
			boolean conn = true;
			try
			{
				String message = null;
				message = (String)input.readObject();
				statusArea.append("\n"+message);
				StringTokenizer x = new StringTokenizer(message, ",:");
				// Client :
				x.nextToken();x.nextToken();
				// Agent
				x.nextToken();

				// finding the avataar. Robot/Human/S_wAgent
				avataar = x.nextToken();
				System.out.println(avataar);

				// This has the information on connection
				ConInfo ci = new ConInfo();

				// If the requested avatar is Robot
				if(avataar.equals("Robot"))
				{
					// Get the total number of robots available
					int robotCount = rbDetails.getRobotCount();
					System.out.println("Total Robot count = " + rbDetails.getRobotCount());
					System.out.println();
					// if all the available robots have been assigned,
					// then send a message that no robots are available
					if(nRobotsAvail==robotCount)
					{
						ci.setMsg("Robots Not Available");
						conn = false;

						output.writeObject(ci);
						output.flush();

						output.close();
						input.close();
					}
				}
				else
				if(avataar.equals("Human"))		// Avatar is Human
				{
					int humanCount = hmnDetails.getHumanCount();
					System.out.println("Total Human Avatar count = " + hmnDetails.getHumanCount());
					if(nHumansAvail==humanCount)
					{
						ci.setMsg("Humans Not Available");
						conn = false;

						output.writeObject(ci);
						output.flush();

						output.close();
						input.close();

					}
				}

				// Starting Location

				// Obtaining the other information like
				//1. starting location
				//2. Orientation of the agent (N,E,S,W)

				x.nextToken();
				int startLoc_x = (int) Double.parseDouble(x.nextToken());
				int startLoc_y = (int) Double.parseDouble(x.nextToken());

				currentPos = new GridPoint(startLoc_x, startLoc_y);
				startingPos = new GridPoint(startLoc_x, startLoc_y);

				x.nextToken();
				orientation = Integer.parseInt(x.nextToken());
				orgOrientation = orientation;
				System.out.println("Agent is Facing " + orientation);

				System.out.println(startLoc_x + ", " + startLoc_y);

				// Determine whether the starting location specified
				// is valid => At that location, there is
				// 1. No Agent
				// 2. Not Deadly(ie, No Pit or Wumpus)
				// 3. No Obstacle
				if(avataar.equals("Human") || avataar.equals("Robot"))
				{
					if(drawingArea.getEnvironment().isDeadly(new GridPoint(startLoc_x, startLoc_y)) ||
					   drawingArea.getEnvironment().isObstacleAt(startLoc_x, startLoc_y) ||
					   drawingArea.getEnvironment().isRRAgentWithR_HAt(new GridPoint(startLoc_x, startLoc_y)))
					{
						ci.setMsg("Not a valid starting location");

						output.writeObject(ci);
						output.flush();
						
						output.close();
						input.close();
						conn = false;
					}
				}
				else
				{
					// avatar is a software entity, then it can exist
					// with another avatar in the same location
					if(drawingArea.getEnvironment().isDeadly(new GridPoint(startLoc_x, startLoc_y)) ||
					   drawingArea.getEnvironment().isObstacleAt(startLoc_x, startLoc_y)) ///|| drawingArea.getEnvironment().isAgentWithR_HAt(new GridPoint(startLoc_x, startLoc_y)))
					{
						ci.setMsg("Not a valid starting location");
						conn = false;

						output.writeObject(ci);
						output.flush();

						output.close();
						input.close();

					}
				}

				if(conn)
				{
					if(avataar.equals("Robot"))
					{

						Vector data = rbDetails.getData();
						Robot robotDetails = (Robot)data.elementAt(nRobotsAvail);
						avataarIP = robotDetails.getRbCIP();
						avataarPort = robotDetails.getRbCPort();
						System.out.println(robotDetails.getImageIP() +", " + robotDetails.getImagePort());
						ci.setIP(new String(robotDetails.getImageIP()));
						ci.setPort(robotDetails.getImagePort()); //(255);
						// decrementing the counter as this one has been assigned
						nRobotsAvail++;

						/*
							Here we have to make sure that we send the robot appropriate
							messages to move to the specified starting location
							and then wait till it gets there.
							After that the game will proceed as usual!
						*/
					}

					if(avataar.equals("Human"))
					{
						/*
						   Do something appropriate here
						   1. Obtain the connection from the human avataar
						   2. Send message to the human avatar to basically move to
						      the specified location by the agent.
						   3. Wait till the avataar gets there
						   4. Increment the count for human avataar assigned to the agents
						*/
						Vector data = hmnDetails.getData();
						Human humanDetails = (Human)data.elementAt(nHumansAvail);
						avataarIP = humanDetails.getRbCIP();
						avataarPort = humanDetails.getRbCPort();
						System.out.println(humanDetails.getImageIP() +", " + humanDetails.getImagePort());
						ci.setIP(new String(humanDetails.getImageIP()));
						ci.setPort(humanDetails.getImagePort());
						// decrementing the counter as this one has been assigned
						nHumansAvail++;
					}


					trialScore = 0;
					nArrows = 10;
					nGoldPieces = 0;
					actionTaken = Action.NO_ACTION;
					currentStep = 1;
					currentTrial = 1;
					trialOver = true;

					scoreBoard = new ScoreKeeper(noofTrials);
					scoreKeeper.add(scoreBoard);

					// Obtain an agent ID number
					clientNumber = getClientNumber();
					scoreBoard.setAgentID(clientNumber);
					// Send a message saying that the request for connection has been
					// accepted and the agent can now receive percepts and start playing
					// the game

					ci.setMsg("Request Accepted Successfully, Round Robin Scheme");
					ci.setID(clientNumber);

					// Setting maximum number of steps per trial
					ci.setMaxStepsPerTrial(maxSteps);
					// Setting number of trials per game
					ci.setNumberOfTrials(noofTrials);

					ci.setTimeoutLimit(timeToAcceptCon+(noofClients*timeToAcceptActions));
					// Object containing all the information has been dispatched
					
					output.writeObject(ci);
					output.flush();


					boolean bGame = true;

					// Useful only when the avatar is robot or human.
					// this basically creates a datagram socket that will be used to
					// send datagram messages to the robot or human.
					// these messages are action or control messages
					if(avataar.equals("Robot") || avataar.equals("Human"))
					{

						try
						{
							// Creating a new datagram socket.
							dgsock = new DatagramSocket();

						}
						catch(SocketException se)
						{
							JOptionPane.showMessageDialog(null, "Error : Unable to create a datagram socket to forward actions to avataar");
							dgsock = null;
						}
					}


					// Creating a moving object that would act as a proxy to all the
					// actions specified by the agent.
					// It will execute the action and see if its valid in the environment
					// If it is valid action, then
					//   if the avataar is robot or human then
   					//         action dispatched to the avataar
   					//   endif
   					// endif
					mo = new MovingObject(1,currentPos.getPtX(), currentPos.getPtY(), orientation);
					wumpusShot = false;

				}

			}
			catch(ClassNotFoundException cnfex)
			{
						conn = false;
			}
			catch(EOFException eof)
			{
						conn = false;
			}
			catch(IOException ioex)
			{
				conn = false;
			}
			catch ( Exception e ) {
				System.out.println ( "zhao xiaoru1 exceptione: " +  e.getMessage () );
				e.printStackTrace ();
			}
			return conn;
		}
		public void resetGameVariables()
		{

			trialScore = 0;
			nArrows = 10;
			nGoldPieces = 0;
			actionTaken = Action.NO_ACTION;
			/*
			 Here we reset the starting position where the agent goes and waits
			 When the avataar is "Robot" and "Human" code should be written that
			 would ask the avataar to go to the starting location from where it is right
			 now.
			*/

			mo = new MovingObject(1,startingPos.getPtX(), startingPos.getPtY(), orientation);
			orientation = orgOrientation;
			currentStep = 1;


		}
		public boolean isTrialOver()
		{
			return trialOver;
		}

		public boolean sendPercept()
		{
			try
			{
					Percept p = new Percept(clientNumber, drawingArea.getEnvironment());


					ConInfo t = new ConInfo();

					p.perceive(new GridPoint(mo.getLocation().getPtX(), mo.getLocation().getPtY()),mo.getOrientation(),t);


					// If a wumpus was shot because of previous action,
					// Send a scream in the percept
					if(wumpusShot == false)
						t.setScream(false);
					else
					{
						t.setScream(true);
						wumpusShot = false;
					}

					// Sets the current Step
					t.setCurrentStep(currentStep);

					// Sets the current Trial
					t.setCurrentTrial(currentTrial);

					// Sets the number of gold pieces collected
					// by the agent so far
					t.setNumGold(nGoldPieces);

					// Sets the total number of arrows remaining
					t.setNumArrow(nArrows);

					// Sets the current score so far
					t.setScore(trialScore);

					// Sets the action last taken

					t.setActionTaken(actionTaken);

					// Sets the state of the agent
					t.setStatus(ActorState.ALIVE);

					// Sets the image of the next cell that the agent
					// is facing
					t.setImage(p.getImagePixels(), p.getImageHeight(),p.getImageWidth());

					// Sets a dummy hint for the agent.
					t.setHint(p.getHint());

					// Object with all the percepts and other information dispatched
					// to the agent.
					trialOver = false;

					output.writeObject(t);
					output.flush();



					if(currentStep > maxSteps)
					{
						currentTrial++;
						// check if the current trial is the last trial
						if(currentTrial > noofTrials)
						{
							disconnectClient(clientNumber);
							agentList.removeElement(this);
						}
					}

			}
			catch(Exception e)
			{
				System.out.println("Error while sending the percept to agent : " + clientNumber);
				disconnectClient(clientNumber);
				agentList.removeElement(this);

				return false;
			}
			return true;
		}
		public boolean receiveAction()
		{

			try
			{
				s1 = (String)input.readObject();
				count = 0;

				System.out.println(" Action Taken by Agent = " + s1);
				actionTaken = Integer.parseInt(s1);
				// Incrementing the step per trial
				currentStep++;
			}
			catch(ClassNotFoundException cnfx)
			{
				disconnectClient(clientNumber);
				agentList.removeElement(this);
				return false;
			}
			catch(NumberFormatException e1)
			{
				actionTaken = Action.SIT;
				count++;
				if(count>=2)
				{

					/*Failed to obtain */
					disconnectClient(clientNumber);
					agentList.removeElement(this);

					return false;
				}

			}
			catch(InterruptedIOException e)
			{
				System.out.println("Did not obtain action from agent within time limit!");

				actionTaken = Action.SIT;
				count++;
				if(count>=2)
				{

					disconnectClient(clientNumber);
					agentList.removeElement(this);
					try
					{
						output.writeObject("Disconnecting");
						output.flush();
					}
					catch(IOException e1)
					{
						System.out.println ( "zhao xiaoru2 " + e1.getMessage () );
						e1.printStackTrace ();
					
					}
					/*Failed to obtain */

					return false;
				}
				else
					{
						try
						{
							output.writeObject("Did not receive action");
							output.flush();
						}
						catch(IOException e2)
						{
							System.out.println ( "zhao xiaoru3 " + e2.getMessage () );
							e2.printStackTrace ();
						}
					}
				return false;
			}
			catch(IOException e2)
			{
					disconnectClient(clientNumber);
					agentList.removeElement(this);
					return false;
			}
			return true;
		}
		public boolean executeAction()
		{
			try
			{
				if(Integer.parseInt(s1)==Action.GOFORWARD)
				{
					actionTaken = Action.GOFORWARD;
					MovingObject dummy = new MovingObject(1,mo.getLocation().getPtX(), mo.getLocation().getPtY(), mo.getOrientation());
					dummy.move();
					GridPoint gp = dummy.getLocation();

					// This means, that the you took the action to move
					// and then the score is decremented by 1
					trialScore--;
					scoreBoard.setScoreForTrial(trialScore, currentTrial);

					// Agent moves forward
					// Check if the location is valid and also, if there is another
					// agent at that location whose avatar is "Robot" or "Human"

					// If the location is valid and does not have robot or human avataar
					// then the move is valid
					if(drawingArea.getEnvironment().isValid(gp) &&  (!(drawingArea.getEnvironment().isRRAgentWithR_HAt(gp)) || avataar.equals("s/w Avataar")) )
					{
						// Setting the score because, irrespective of
						// whether the trial is over or not

						if(drawingArea.getEnvironment().isDeadly(gp))
						{
							output.writeObject("Game Over");
							output.flush();


							// Incrementing the trial count

							currentTrial++;


							ConInfo t1 = new ConInfo();
							t1.setActionTaken(actionTaken);
							t1.setHint("You have climbed out of the Wumpus World");
							// Sets the state of the agent
							t1.setStatus(ActorState.ALIVE);
							t1.setCurrentStep(currentStep);
							t1.setScore(trialScore);

							output.writeObject(t1);
							output.flush();


							if(currentTrial > noofTrials)
							{

								disconnectClient(clientNumber);
								agentList.removeElement(this);
								//continue;
								/*
									Game is over now
								*/
							}
							else
							{
								/*
									Current trial got over. Prepare for the
									next trial

									Since the game can get over here at the position
									where the agent may not be at the starting location
									then ask the robot or the human avatar to move to
									the	appropriate starting location having the
									original orientation

								*/
								trialOver = true;
								resetGameVariables();
								//startNextTrial();
								//continue;
							}

						}
						else
						{
							/* Valid Move */
							mo.move();
							output.writeObject("Success");
							output.flush();
							output.writeObject(mo.getLocation().getPtX()+ " " + mo.getLocation().getPtY()+" "+mo.getOrientation());
							output.flush();
						}

						repaint();
					}
					else
					{
						output.writeObject("Invalid Location");
						output.flush();
					}

				}
				else
				if(Integer.parseInt(s1)==Action.TURN_LEFT)
				{
					actionTaken = Action.TURN_LEFT;
					mo.turnLeft();
					output.writeObject("Success");
					output.flush();
					output.writeObject( mo.getLocation().getPtX()+ " " + mo.getLocation().getPtY()+" "+mo.getOrientation());
					output.flush();

					// Adjusting the score
					trialScore--;
					scoreBoard.setScoreForTrial(trialScore, currentTrial);

				}
				else
				if(Integer.parseInt(s1)==Action.TURN_RIGHT)
				{
					actionTaken = Action.TURN_RIGHT;
					mo.turnRight();
					output.writeObject("Success");
					output.flush();
					output.writeObject(mo.getLocation().getPtX()+ " " + mo.getLocation().getPtY()+" "+mo.getOrientation());
					output.flush();

					// Adjusting the score
					trialScore--;
					scoreBoard.setScoreForTrial(trialScore, currentTrial);


				}
				else
				if(Integer.parseInt(s1)==Action.GRAB)
				{
					//t.setActionTaken(Action.GRAB);
					actionTaken = Action.GRAB;
					if(drawingArea.getEnvironment().isGoldAt(mo.getLocation().getPtX(), mo.getLocation().getPtY()))
					{
						/*
							Add 1000 points to the scorer
						*/
						drawingArea.getEnvironment().removeGoldAt(new GridPoint(mo.getLocation().getPtX(), mo.getLocation().getPtY()));
						trialScore += 1000;
						scoreBoard.setScoreForTrial(trialScore, currentTrial);

						nGoldPieces++;
						output.writeObject("Success");
						output.flush();
						output.writeObject(mo.getLocation().getPtX()+ " " + mo.getLocation().getPtY()+" "+mo.getOrientation());
						output.flush();


					}
					else
					{
						output.writeObject("No Gold Found");
						output.flush();
					}
				}
				else
				if(Integer.parseInt(s1)==Action.SIT)
				{
					//t.setActionTaken(Action.SIT);
					actionTaken = Action.SIT;

					// Adjusting the score
					trialScore--;
					scoreBoard.setScoreForTrial(trialScore, currentTrial);

					output.writeObject("Success");
					output.flush();
					output.writeObject(mo.getLocation().getPtX()+ " " + mo.getLocation().getPtY()+" "+mo.getOrientation());
					output.flush();
				}
				else
				if(Integer.parseInt(s1)==Action.CLIMB)
				{
					//t.setActionTaken(Action.CLIMB);
					currentStep++;
					actionTaken = Action.CLIMB;
					if(startingPos.equals(mo.getLocation()))
					{
						// Adjusting the score. Successfully climbed out
						trialScore+= 10000;
						scoreBoard.setScoreForTrial(trialScore, currentTrial);

						output.writeObject("Climbed Out");
						output.flush();
						currentTrial++;

						ConInfo t1 = new ConInfo();
						t1.setActionTaken(actionTaken);
						t1.setHint("You have climbed out of the Wumpus World");
						// Sets the state of the agent
						t1.setStatus(ActorState.ALIVE);
						t1.setCurrentStep(currentStep);
						t1.setScore(trialScore);

						output.writeObject(t1);
						output.flush();

						// Quitting the agent thread after
						// exhausting all the trials.
						if(currentTrial > noofTrials)
						{
							disconnectClient(clientNumber);
							agentList.removeElement(this);
						}
						else
						{
							/*
								Prepare for the next trial here.
								since the trial was successful, the agent is at the same location
								as the starting location.

								Write code to reset the orienation of the avatar to the
								original orientation specified by the agent.

							*/

							trialOver = true;
							resetGameVariables();

						}
					}
					else
					{
						// Adjusting the score. Could not climb out of the
						// wumpus world.
						trialScore--;
						scoreBoard.setScoreForTrial(trialScore, currentTrial);

						output.writeObject("Not starting location");
						output.flush();

					}

				}
				else
				if(Integer.parseInt(s1)==Action.SHOOT)
				{
					actionTaken = Action.SHOOT;
					if(nArrows>0)
					{
						trialScore--;
						scoreBoard.setScoreForTrial(trialScore, currentTrial);

						nArrows--;
						if(drawingArea.getEnvironment().shootAtWumpus(mo.getLocation(), mo.getOrientation()))
						{
							wumpusShot = true;
							output.writeObject("Shot Wumpus");
							output.flush();
						}
						else
						{
							output.writeObject("No Wumpus Shot");
							output.flush();
						}
					}
					else
					{
						output.writeObject("Out of Arrows");
						output.flush();
					}
				}
				else
				{
					// Just for extension purposes to add some more actions

					output.writeObject("Not Implemented");
					output.flush();
					trialScore--;
					scoreBoard.setScoreForTrial(trialScore, currentTrial);


				}

				// If the dategram socket is  not null
				// and if the avataar is a robot or a human then send the action
				// across to the avataar so that the avataar can move and
				// update the position
				if(dgsock!=null && (avataar.equals("Robot") || avataar.equals("Human")))
				{
					byte data[] = s1.getBytes();
					DatagramPacket sendData = new DatagramPacket(data,data.length, InetAddress.getByName(avataarIP),avataarPort);

					// Sending data to the avataar in wumpus world to execute the instruction
					dgsock.send(sendData);
				}



			}
			catch(IOException e)
			{
				disconnectClient(clientNumber);
				agentList.removeElement(this);
				return false;

			}
			return true;
		}

		public MovingObject getCurrentLocation()
		{
			return mo;
		}
		public String getAvataar()
		{
			return avataar;
		}

	}

	class AgentThread extends Thread
	{
		private Monitor monitor;
		private Socket socket;
		private ObjectInputStream input;
		private ObjectOutputStream output;

		private int clientNumber;
		private GridPoint gp;

		private boolean suffRsc;
		private String avataar;
		private String avataarIP;
		private int    avataarPort;

		private GridPoint startingPos;

		private GridPoint currentPos;

		private int trialScore;
		private int nArrows;
		private int nGoldPieces;
		private int actionTaken;

		private int currentStep;
		private int currentTrial;

		private boolean trialOver;

		private boolean status;

		private Thread t;

		private MovingObject mo;
		private int orgOrientation;
		private int orientation;
		private boolean state;
		private ScoreKeeper scoreBoard;



		public void resetGameVariables()
		{

			trialScore = 0;
			nArrows = 10;
			nGoldPieces = 0;
			actionTaken = Action.SIT;
			/*
			 Here we reset the starting position where the agent goes and waits
			 When the avataar is "Robot" and "Human" code should be written that
			 would ask the avataar to go to the starting location from where it is right
			 now.
			*/
		    mo = new MovingObject(1,startingPos.getPtX(), startingPos.getPtY(), orientation);
			orientation = orgOrientation;
			currentStep = 1;


		}


		public MovingObject getCurrentLocation()
		{
			return mo;
		}

		// Returns the state of a thread, if it is waiting
		// or running
		public boolean getState()
		{
			return state;
		}
		public AgentThread(Socket s, boolean st, Monitor m)
		{
			socket = s;
			monitor = m;

			trialScore = 0;
			nArrows = 10;
			nGoldPieces = 0;
			actionTaken = Action.NO_ACTION;
			currentStep = 1;
			currentTrial = 1;
			trialOver = false;
			scoreBoard = new ScoreKeeper(noofTrials);
			scoreKeeper.add(scoreBoard);
			mo = null;

			try
			{
				b_Terminate = false;
				suffRsc = true;
				trialOver = false;
				status = st;
				gp = new GridPoint();

				/* Time out interval is now 3 minutes */
				socket.setSoTimeout(timeToAcceptActions);
				input = new ObjectInputStream(socket.getInputStream());
				output = new ObjectOutputStream(socket.getOutputStream());

				t = new Thread(this);
				t.start();
			}
			catch(EOFException eof)
			{
				System.out.println ("zhaoxiaoru4: " + eof.getMessage () );
				eof.printStackTrace ();
			}
			catch(IOException ioex)
			{
				System.out.println ("zhaoxiaoru4: " + ioex.getMessage () );
				ioex.printStackTrace ();

			}

		}
		private boolean b_Terminate = false;

		public void setTerminate()
		{
			b_Terminate = true;
		}

		public boolean isTrialOver()
		{
			return trialOver;
		}

		public synchronized int getCurrentTrial()
		{
			return currentTrial;
		}

		private void startNextTrial()
		{
		  try
		  {
			trialOver = true;
			socket.setSoTimeout(maxSteps*timeToAcceptActions);
			// Reset the game variables as the agent will play the
			// next trial!
			resetGameVariables();
			repaint();

			/* Waits for the other agents to finish their trial or wakes other
			   agents up if it is the last one
			*/
			if(monitor.hit())
			{


			}
			trialOver = false;
			socket.setSoTimeout(timeToAcceptActions);
	      }
	      catch(SocketException s)
	      {
				System.out.println ("zhaoxiaoru4: " + s.getMessage () );
				s.printStackTrace ();
		  }
		}

		public String getAvataar()
		{
			return avataar;
		}
		public void run()
		{

			System.out.println("The agent thread is running!");
				// Get the message from the agent.
				// This gives the type of agent and type of avatar requested
			try
			{
				String message = null;
				message = (String)input.readObject();
				statusArea.append("\n"+message);
				StringTokenizer x = new StringTokenizer(message, ",:");
				// Client :
				x.nextToken();x.nextToken();
				// Agent
				x.nextToken();

				// finding the avataar. Robot/Human/S_wAgent
				avataar = x.nextToken();
				System.out.println(avataar);

				// This has the information on connection
				ConInfo ci = new ConInfo();

				// If the requested avatar is Robot
				if(avataar.equals("Robot"))
				{
					// Get the total number of robots available
					int robotCount = rbDetails.getRobotCount();
					System.out.println("Total Robot count = " + rbDetails.getRobotCount());
					System.out.println();
					// if all the available robots have been assigned,
					// then send a message that no robots are available
					if(nRobotsAvail==robotCount)
					{
						ci.setMsg("Robots Not Available");
					
						output.writeObject(ci);
						output.flush();

						output.close();
						input.close();

						// Sufficient resources are not available to
						// accept the request
						suffRsc = false;
					}
				}
				else
				if(avataar.equals("Human"))		// Avatar is Human
				{
					int humanCount = hmnDetails.getHumanCount();
					System.out.println("Total Human Avatar count = " + hmnDetails.getHumanCount());
					if(nHumansAvail==humanCount)
					{
						ci.setMsg("Humans Not Available");

						output.writeObject(ci);
						output.flush();

						output.close();
						input.close();

						suffRsc = false;
						// sufficient resources are not available to
						// accept the request
					}
				}

				// Starting Location

				// Obtaining the other information like
				//1. starting location
				//2. Orientation of the agent (N,E,S,W)

				x.nextToken();
				int startLoc_x = (int) Double.parseDouble(x.nextToken());
				int startLoc_y = (int) Double.parseDouble(x.nextToken());

				currentPos = new GridPoint(startLoc_x, startLoc_y);
				startingPos = new GridPoint(startLoc_x, startLoc_y);

				x.nextToken();
				orientation = Integer.parseInt(x.nextToken());
				orgOrientation = orientation;
				System.out.println("Agent is Facing " + orientation);

				System.out.println(startLoc_x + ", " + startLoc_y);

				// Determine whether the starting location specified
				// is valid => At that location, there is
				// 1. No Agent
				// 2. Not Deadly(ie, No Pit or Wumpus)
				// 3. No Obstacle
				if(avataar.equals("Human") || avataar.equals("Robot"))
				{
					if(drawingArea.getEnvironment().isDeadly(new GridPoint(startLoc_x, startLoc_y)) ||
					   drawingArea.getEnvironment().isObstacleAt(startLoc_x, startLoc_y) ||
					   drawingArea.getEnvironment().isAgentWithR_HAt(new GridPoint(startLoc_x, startLoc_y)))
					{
						ci.setMsg("Not a valid starting location");

						output.writeObject(ci);
						output.flush();
						
						output.close();
						input.close();
					}
				}
				else
				{
					// avatar is a software entity, then it can exist
					// with another avatar in the same location
					if(drawingArea.getEnvironment().isDeadly(new GridPoint(startLoc_x, startLoc_y)) ||
					   drawingArea.getEnvironment().isObstacleAt(startLoc_x, startLoc_y)) ///|| drawingArea.getEnvironment().isAgentWithR_HAt(new GridPoint(startLoc_x, startLoc_y)))
					{
						ci.setMsg("Not a valid starting location");

						output.writeObject(ci);
						output.flush();

						output.close();
						input.close();
					}
				}
				// Check if the game is already in progress, ie
				// if the required number of agents have logged onto the
				// wumpus world server and the game has already started


				if(status)
				{
					ci.setMsg("Game in progress");
		
					output.writeObject(ci);
					output.flush();

					output.close();
					input.close();
				}
				else
				{
				  // Start the game only if there are only enough resources
				  // ie, the avataar, robot or human requested is available
				  if(suffRsc)
				  {

					if(avataar.equals("Robot"))
					{

						Vector data = rbDetails.getData();
						Robot robotDetails = (Robot)data.elementAt(nRobotsAvail);
						avataarIP = robotDetails.getRbCIP();
						avataarPort = robotDetails.getRbCPort();
						System.out.println(robotDetails.getImageIP() +", " + robotDetails.getImagePort());
						ci.setIP(new String(robotDetails.getImageIP()));
						ci.setPort(robotDetails.getImagePort());
						// decrementing the counter as this one has been assigned
						nRobotsAvail++;

						/*
							Here we have to make sure that we send the robot appropriate
							messages to move to the specified starting location
							and then wait till it gets there.
							After that the game will proceed as usual!
						*/
					}

					if(avataar.equals("Human"))
					{
						/*
						   Do something appropriate here
						   1. Obtain the connection from the human avataar
						   2. Send message to the human avatar to basically move to
						      the specified location by the agent.
						   3. Wait till the avataar gets there
						   4. Increment the count for human avataar assigned to the agents
						*/
						Vector data = hmnDetails.getData();
						Human humanDetails = (Human)data.elementAt(nHumansAvail);
						avataarIP = humanDetails.getRbCIP();
						avataarPort = humanDetails.getRbCPort();
						System.out.println(humanDetails.getImageIP() +", " + humanDetails.getImagePort());
						ci.setIP(new String(humanDetails.getImageIP()));
						ci.setPort(humanDetails.getImagePort());
						// decrementing the counter as this one has been assigned
						nHumansAvail++;
					}

					// Obtain an agent ID number
					clientNumber = getClientNumber();
					scoreBoard.setAgentID(clientNumber);
					// Send a message saying that the request for connection has been
					// accepted and the agent can now receive percepts and start playing
					// the game

					ci.setMsg("Request Accepted Successfully");
					ci.setID(clientNumber);

					// Setting maximum number of steps per trial
					ci.setMaxStepsPerTrial(maxSteps);
					// Setting number of trials per game
					ci.setNumberOfTrials(noofTrials);

					// Object containing all the information has been dispatched

					output.writeObject(ci);
					output.flush();



					boolean bGame = true;

					DatagramSocket dgsock=null;
					// Useful only when the avatar is robot or human.
					// this basically creates a datagram socket that will be used to
					// send datagram messages to the robot or human.
					// these messages are action or control messages
					if(avataar.equals("Robot") || avataar.equals("Human"))
					{

						try
						{
							// Creating a new datagram socket.
							dgsock = new DatagramSocket();

						}
						catch(SocketException se)
						{
							JOptionPane.showMessageDialog(null, "Error : Unable to create a datagram socket to forward actions to avataar");
							dgsock = null;
						}
					}


					// Creating a moving object that would act as a proxy to all the
					// actions specified by the agent.
					// It will execute the action and see if its valid in the environment
					// If it is valid action, then
					//   if the avataar is robot or human then
   					//         action dispatched to the avataar
   					//   endif
   					// endif
					mo = new MovingObject(1,currentPos.getPtX(), currentPos.getPtY(), orientation);
					boolean wumpusShot = false;
					//wP.TransferPercept t = new wP.TransferPercept();

					// This loop will run as long as bGame is true
					// ie, as long as the game is in progress
					agentThreads.add(this);
					while(bGame)
					{
						// Sends out the percept at the starting location specified
						// by the agent
						Percept p = new Percept(clientNumber, drawingArea.getEnvironment());


						ConInfo t = new ConInfo();

						p.perceive(new GridPoint(mo.getLocation().getPtX(), mo.getLocation().getPtY()),mo.getOrientation(),t);


						// If a wumpus was shot because of previous action,
						// Send a scream in the percept
						if(wumpusShot == false)
							t.setScream(false);
						else
						{
							t.setScream(true);
							wumpusShot = false;
						}

						// Sets the current Step
						t.setCurrentStep(currentStep);

						// Sets the current Trial
						t.setCurrentTrial(currentTrial);

						// Sets the number of gold pieces collected
						// by the agent so far
						t.setNumGold(nGoldPieces);

						// Sets the total number of arrows remaining
						t.setNumArrow(nArrows);

						// Sets the current score so far
						t.setScore(trialScore);

						// Sets the action last taken

						t.setActionTaken(actionTaken);

						// Sets the state of the agent
						t.setStatus(ActorState.ALIVE);

						// Sets the image of the next cell that the agent
						// is facing
//						t.setImage(p.getImagePixels(), p.getImageHeight(),p.getImageWidth());
						ImageTransmitter it = new ImageTransmitter (); //zhao
						int h = p.getImageHeight ();
						int w = p.getImageWidth ();
						t.setImageHeight ( h ); //zhao
						t.setImageWidth ( w );  //zhao
						it.transmitImage ( p.getImagePixels (), h, w ); //zhao

						// Sets a dummy hint for the agent.
						t.setHint(p.getHint());

						// Object with all the percepts and other information dispatched
						// to the agent.

						output.writeObject(t);
						output.flush();


						if(currentStep > maxSteps)
						{
							currentTrial++;
							// check if the current trial is the last trial
							if(currentTrial > noofTrials)
							{
								// end the game since the required number of trials
								// is over
								bGame = false;
								continue;
							}
							// Start the next trial
							// The agent may have to wait for other agents
							// to finish their trial.
							startNextTrial();
							continue;
						}

						// Waits for the action taken by the agent
						String s1 = (String)input.readObject();

						System.out.println(" Action Taken by Agent = " + s1);

						// Incrementing the step per trial
						currentStep++;
						/*
							Here, I have to validate the move and
							if it is a valid move, then forward the action to
							the avataar(Human/Robot)
							Send a result back
						*/

						// Writes whether the action was successful or not!
						if(Integer.parseInt(s1)==Action.GOFORWARD)
						{
							//t.setActionTaken(Action.GOFORWARD);
							actionTaken = Action.GOFORWARD;
							MovingObject dummy = new MovingObject(1,mo.getLocation().getPtX(), mo.getLocation().getPtY(), mo.getOrientation());
							dummy.move();
							GridPoint gp = dummy.getLocation();

							// This means, that the you took the action to move
							// and then the score is decremented by 1
							trialScore--;
							scoreBoard.setScoreForTrial(trialScore, currentTrial);

							// Agent moves forward
							// Check if the location is valid and also, if there is another
							// agent at that location whose avatar is "Robot" or "Human"

							// If the location is valid and does not have robot or human avataar
							// then the move is valid
							if(drawingArea.getEnvironment().isValid(gp) &&  (!(drawingArea.getEnvironment().isAgentWithR_HAt(gp)) || avataar.equals("s/w Avataar")) )
							{
								// Setting the score because, irrespective of
								// whether the trial is over or not

								if(drawingArea.getEnvironment().isDeadly(gp))
								{
									output.writeObject("Game Over");
									output.flush();


									// Incrementing the trial count

									currentTrial++;


									ConInfo t1 = new ConInfo();
									t1.setActionTaken(actionTaken);
									t1.setHint("You have climbed out of the Wumpus World");
									// Sets the state of the agent
									t1.setStatus(ActorState.ALIVE);
									t1.setCurrentStep(currentStep);
									t1.setScore(trialScore);

									output.writeObject(t1);
									output.flush();


									if(currentTrial > noofTrials)
									{
										bGame = false;
										continue;
										/*
											Game is over now
										*/
									}
									else
									{
										/*
											Current trial got over. Prepare for the
											next trial

											Since the game can get over here at the position
											where the agent may not be at the starting location
											then ask the robot or the human avatar to move to
											the	appropriate starting location having the
											original orientation

										*/

										startNextTrial();
										continue;
									}

								}
								else
								{
									/* Valid Move */
									mo.move();
									output.writeObject("Success");
									output.flush();
									output.writeObject(mo.getLocation().getPtX()+ " " + mo.getLocation().getPtY()+" "+mo.getOrientation());
									output.flush();
								}

								repaint();
							}
							else
							{
								output.writeObject("Invalid Location");
								output.flush();
							}

						}
						else
						if(Integer.parseInt(s1)==Action.TURN_LEFT)
						{
							//t.setActionTaken(Action.TURN_LEFT);
							actionTaken = Action.TURN_LEFT;
							mo.turnLeft();
							output.writeObject("Success");
							output.flush();
							output.writeObject( mo.getLocation().getPtX()+ " " + mo.getLocation().getPtY()+" "+mo.getOrientation());
							output.flush();

							// Adjusting the score
							trialScore--;
							scoreBoard.setScoreForTrial(trialScore, currentTrial);

						}
						else
						if(Integer.parseInt(s1)==Action.TURN_RIGHT)
						{
							//t.setActionTaken(Action.TURN_RIGHT);
							actionTaken = Action.TURN_RIGHT;
							mo.turnRight();
							output.writeObject("Success");
							output.flush();
							output.writeObject(mo.getLocation().getPtX()+ " " + mo.getLocation().getPtY()+" "+mo.getOrientation());
							output.flush();

							// Adjusting the score
							trialScore--;
							scoreBoard.setScoreForTrial(trialScore, currentTrial);


						}
						else
						if(Integer.parseInt(s1)==Action.GRAB)
						{
							//t.setActionTaken(Action.GRAB);
							actionTaken = Action.GRAB;
							if(drawingArea.getEnvironment().isGoldAt(mo.getLocation().getPtX(), mo.getLocation().getPtY()))
							{
								/*
									Add 1000 points to the scorer
								*/
								// Removing the gold from the wumpus world
								drawingArea.getEnvironment().removeGoldAt(new GridPoint(mo.getLocation().getPtX(), mo.getLocation().getPtY()));
								trialScore += 1000;
								scoreBoard.setScoreForTrial(trialScore, currentTrial);

								nGoldPieces++;
								output.writeObject("Success");
								output.flush();
								output.writeObject(mo.getLocation().getPtX()+ " " + mo.getLocation().getPtY()+" "+mo.getOrientation());
								output.flush();


							}
							else
							{
								output.writeObject("No Gold Found");
								output.flush();
							}
						}
						else
						if(Integer.parseInt(s1)==Action.SIT)
						{
							//t.setActionTaken(Action.SIT);
							actionTaken = Action.SIT;

							// Adjusting the score
							trialScore--;
							scoreBoard.setScoreForTrial(trialScore, currentTrial);

							output.writeObject("Success");
							output.flush();
							output.writeObject(mo.getLocation().getPtX()+ " " + mo.getLocation().getPtY()+" "+mo.getOrientation());
							output.flush();
						}
						else
						if(Integer.parseInt(s1)==Action.CLIMB)
						{
							//t.setActionTaken(Action.CLIMB);
							currentStep++;
							actionTaken = Action.CLIMB;
							if(startingPos.equals(mo.getLocation()))
							{
								// Adjusting the score. Successfully climbed out
								trialScore+= 10000;
								scoreBoard.setScoreForTrial(trialScore, currentTrial);

								output.writeObject("Climbed Out");
								output.flush();
								currentTrial++;
								ConInfo t1 = new ConInfo();
								t1.setActionTaken(actionTaken);
								t1.setHint("You have climbed out of the Wumpus World");
								// Sets the state of the agent
								t1.setStatus(ActorState.ALIVE);
								t1.setCurrentStep(currentStep);
								t1.setScore(trialScore);

								output.writeObject(t1);
								output.flush();

								// Quitting the agent thread after
								// exhausting all the trials.
								if(currentTrial > noofTrials)
								{
									bGame = false;
									continue;
									// This will make sure that the game is stopped
								}
								else
								{
									/*
										Prepare for the next trial here.
										since the trial was successful, the agent is at the same location
										as the starting location.

										Write code to reset the orienation of the avatar to the
										original orientation specified by the agent.

									*/
									startNextTrial();
									continue;
								}
							}
							else
							{
								// Adjusting the score. Could not climb out of the
								// wumpus world.
								trialScore--;
								scoreBoard.setScoreForTrial(trialScore, currentTrial);

								output.writeObject("Not starting location");
								output.flush();
								//output.writeObject(mo.getLocation().getPtX()+ " " + mo.getLocation().getPtY()+" "+mo.getOrientation());
								//output.flush();

							}

						}
						else
						if(Integer.parseInt(s1)==Action.SHOOT)
						{
							//t.setActionTaken(Action.SHOOT);
							actionTaken = Action.SHOOT;
							if(nArrows>0)
							{
								trialScore--;
								scoreBoard.setScoreForTrial(trialScore, currentTrial);

								nArrows--;
								if(drawingArea.getEnvironment().shootAtWumpus(mo.getLocation(), mo.getOrientation()))
								{
									wumpusShot = true;
									output.writeObject("Shot Wumpus");
									output.flush();
								}
								else
								{
									output.writeObject("No Wumpus Shot");
									output.flush();
								}
							}
							else
							{
								output.writeObject("Out of Arrows");
								output.flush();
							}
						}
						else
						{
							// Just for extension purposes to add some more actions

							output.writeObject("Not Implemented");
							output.flush();
							trialScore--;
							scoreBoard.setScoreForTrial(trialScore, currentTrial);


						}

						// If the dategram socket is  not null
						// and if the avataar is a robot or a human then send the action
						// across to the avataar so that the avataar can move and
						// update the position
						if(dgsock!=null && (avataar.equals("Robot") || avataar.equals("Human")))
						{
							byte data[] = s1.getBytes();
							DatagramPacket sendData = new DatagramPacket(data,data.length, InetAddress.getByName(avataarIP),avataarPort);

							// Sending data to the avataar in wumpus world to execute the instruction
							dgsock.send(sendData);
						}

					}	//  end of while loop
			       } // End of if(suffResc) statement
			       else
			       {
					   // Connection rejected because of lack of resources
					   //
					   statusArea.append("\nConnection request from Agent : " + clientNumber + " is rejected due to lack of resources");


				   }
				}	// End of else statement for the if(status) command block

				// Closing the input and output sockets that are used to connect to the
				// agent.
				// Yet to close the datagram socket opened to communicate with the
				// avataar
				input.close();
				output.close();
			} // End of the try block

			catch(java.io.InterruptedIOException ie)
			{
				JOptionPane.showMessageDialog(null, "Agent : "+ clientNumber +" Inactive... disconnecting ");
				// Since this occurred because of a time out,
				// We have to call the lock on the timer so that the others who
				// were waiting for this lock can start their next trial
				// Lets say 5 agents were playing the game of which 4 are waiting
				// and the fifth one is now disconnected has to invoke the

				// Remove this agent from the list of agents that are playing the
				// game
				removeAgent(this, clientNumber);

				// Set the client number so that it can be re-used
				//disconnectClient(clientNumber);

				// Call on the monitor to release the lock.
				// if 4 agents are in the game and three are already done with their
				// trial and the last one just disconnects, then we have to invoke the
				// lock on monitor to notifyAll the threads who are waiting on this monitor

				// Repaint because the agent has got disconnected, it should not be
				// displayed in the wumpus world
				repaint();

				// To wake up any of the agents waiting on the other agents
				// to finish the game
				monitor.hit();

				// Repaint so that the agents who got notified may restart their game.
				repaint();
				// Closing the sockets because they are no longer needed
			    try
			    {
					input.close();
					output.close();
				}
				catch(IOException e)
				{
				 	System.out.println ( "zhaoxiaoru5 " + e.getMessage () );
					e.printStackTrace ();
				}
				// Return from the run method because the agent is done with the
				// game

				return;
			}
			catch(NumberFormatException nfe)
			{
				 	System.out.println ( "zhaoxiaoru6 " + nfe.getMessage () );
					nfe.printStackTrace ();

			}
			catch(ClassNotFoundException cnfex)
			{
				 	System.out.println ( "zhaoxiaoru6 " + cnfex.getMessage () );
					cnfex.printStackTrace ();

			}
			catch(EOFException eof)
			{
				 	System.out.println ( "zhaoxiaoru6 " + eof.getMessage () );
					eof.printStackTrace ();
			}
			catch(IOException ioex)
			{
				 	System.out.println ( "zhaoxiaoru6 " + ioex.getMessage () );
					ioex.printStackTrace ();

			}

			// Disconnecting the client
			removeAgent(this, clientNumber);
			//disconnectClient(clientNumber);
		}	// End of run method

	}	// End of class AgentThread

	private class ConfigureDialog extends JDialog
	{
		private JTextField    tf_noofTrials;
		private JTextField    tf_maxSteps;
		private JTextField    tf_noofClients;
		private JTextField    tf_portNumber;
		private JButton       btn_Ok;
		private JButton       btn_Cancel;
		private JRadioButton  rb_fcfs;
		private	JRadioButton  rb_roundRobin;
		private JTextField    tf_conAccTime;

		public ConfigureDialog(JFrame frame, boolean modal)
		{
			super(frame, modal);
			setTitle("Configure Wumpus World Server");
			getContentPane().setLayout(null);
			JLabel lbl_noofTrial = new JLabel("Number of trials : ");
			lbl_noofTrial.setBounds(25,25, 200, 20);
			getContentPane().add(lbl_noofTrial);

			tf_noofTrials = new JTextField();
			tf_noofTrials.setText(Integer.toString(noofTrials));
			tf_noofTrials.setBounds(260,25, 50, 20);
			getContentPane().add(tf_noofTrials);

			JLabel lbl_maxSteps  = new JLabel("Number of steps per Trial : ");
			lbl_maxSteps.setBounds(25, 60, 185, 20);
			getContentPane().add(lbl_maxSteps);

			tf_maxSteps   = new JTextField();
			tf_maxSteps.setText(Integer.toString(maxSteps));
			tf_maxSteps.setBounds(260, 60, 50, 20);
			getContentPane().add(tf_maxSteps);


			JLabel lbl_noofClients = new JLabel("Required number of Agents : ");
			lbl_noofClients.setBounds(25, 90, 185, 20);
			getContentPane().add(lbl_noofClients);

			tf_noofClients = new JTextField();
			tf_noofClients.setText(Integer.toString(noofClients));
			tf_noofClients.setBounds(260, 90, 50, 20);
			getContentPane().add(tf_noofClients);

			rb_roundRobin = new JRadioButton("Round Robin Message Handling");

			rb_roundRobin.setBounds(25, 150, 225, 20);
			getContentPane().add(rb_roundRobin);

			rb_fcfs   = new JRadioButton("Fcfs Message Handling");

			rb_fcfs.setBounds(25, 180, 225, 20);
			getContentPane().add(rb_fcfs);

			JLabel lbl_conAccTime = new JLabel("Time to accept connections (in min) : ");
			lbl_conAccTime.setBounds(20,205, 240, 20);

			tf_conAccTime = new JTextField();
			tf_conAccTime.setBounds(260, 205, 50, 20);
			// Setting time in minutes
			tf_conAccTime.setText("" + (timeToAcceptCon/(60*1000)));
			getContentPane().add(lbl_conAccTime);
			getContentPane().add(tf_conAccTime);



			if(b_roundRobin)
			{
				rb_roundRobin.setSelected(true);
			}
			else
			//if(!b_fcfs)
			//{
				rb_fcfs.setSelected(true);
			//}
			ButtonGroup btnGroup = new ButtonGroup();
			btnGroup.add(rb_roundRobin);

			btnGroup.add(rb_fcfs);
			//rb_fcfs.setSelected(true);

			btn_Ok = new JButton("OK");
			btn_Ok.setBounds(75, 240, 75, 25);
			getContentPane().add(btn_Ok);

			btn_Ok.addActionListener(
				new ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{

						try
						{
							int nC = Integer.parseInt(tf_noofClients.getText());
							int mS = Integer.parseInt(tf_maxSteps.getText());
							int nT = Integer.parseInt(tf_noofTrials.getText());
							int pN = Integer.parseInt(tf_portNumber.getText());
							int tC = Integer.parseInt(tf_conAccTime.getText());

							if((nC<=0)||(mS<=0)||(nT<=0)||(pN<=0) || (tC <= 0))
							{
								throw new Exception();
							}
							noofClients = nC;
							maxSteps = mS;
							noofTrials = nT;
							portNumber = pN;

							//	We get the time in minutes. We have to convert it in microseconds
							timeToAcceptCon = tC*1000*60;

							if(rb_fcfs.isSelected())
							{
								b_fcfs=true;
								b_roundRobin = false;
							}
							else
							{
								if(rb_roundRobin.isSelected())
								{
									b_roundRobin = true;
									b_fcfs = false;
								}
							}

							hide();
						}
						catch(NumberFormatException nfe)
						{
							JOptionPane.showMessageDialog(null, "Please enter only positive integer value");
						}
						catch(Exception e1)
						{
							JOptionPane.showMessageDialog(null, "Values for max Trials, noof clients, Port Number, max Steps cannot be negative or equal to zero...");
						}

					}
				}
				);
			JLabel lblPort = new JLabel("Port number of server : ");
			lblPort.setBounds(25, 120, 225, 20);
			tf_portNumber = new JTextField();
			tf_portNumber.setText(Integer.toString(portNumber));
			tf_portNumber.setBounds(260, 120, 50,20);

			getContentPane().add(lblPort);
			getContentPane().add(tf_portNumber);

			btn_Cancel = new JButton("Cancel");
			btn_Cancel.setBounds(225, 240, 75, 25);
			btn_Cancel.addActionListener(
				new ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						hide();
					}
				}
				);
			getContentPane().add(btn_Cancel);
			pack();
			validate();
			//setSize(300, 200);
		}

		public void setOKDisabled()
		{
			btn_Ok.setEnabled(false);
		}

	}
}		// End of class WWServer

class WumpusModeDialog extends JDialog
{
	String stayString = "Stay";
	String spinString = "Spin";
	String walkString = "Walk";
	String goto_GoldString = "Go To Gold";
	String goto_PitString =  "Go To Pit";
	String randomString   = "Random";

	JRadioButton        stayButton;
	JRadioButton        spinButton;
	JRadioButton        walkButton;
	JRadioButton        randomButton;
	JRadioButton        gotoPitButton;
	JRadioButton        gotoGoldButton;
	JButton btn;
	JButton btnHelp;
	JPanel panel, panel1;
    JPanel j;
    JLabel picture;
    private String sel_wumpusMode;

    public String getMode()
    {
		return sel_wumpusMode;
	}

	public WumpusModeDialog(JFrame frame, boolean modal)
	{

		super(frame, modal);
		setTitle("set wumpus mode");

		stayButton = new JRadioButton(stayString);
        stayButton.setMnemonic('S');
        stayButton.setActionCommand(stayString);
        stayButton.setSelected(true);

        spinButton = new JRadioButton(spinString);
        spinButton.setMnemonic('p');
        spinButton.setActionCommand(spinString);

        walkButton = new JRadioButton(walkString);
        walkButton.setMnemonic('W');
        walkButton.setActionCommand(walkString);

        randomButton = new JRadioButton(randomString);
        randomButton.setMnemonic('R');
        randomButton.setActionCommand(randomString);

        gotoPitButton = new JRadioButton(goto_PitString);
        gotoPitButton.setMnemonic('G');
        gotoPitButton.setActionCommand(goto_PitString);

        gotoGoldButton = new JRadioButton(goto_GoldString);
        gotoGoldButton.setMnemonic('P');
        gotoGoldButton.setActionCommand(goto_GoldString);

        // Group the radio buttons.
        ButtonGroup group = new ButtonGroup();
        group.add(stayButton);
        group.add(spinButton);
        group.add(walkButton);
        group.add(randomButton);
        group.add(gotoPitButton);
        group.add(gotoGoldButton);

        RadioListener myListener = new RadioListener();
        stayButton.addActionListener(myListener);
        spinButton.addActionListener(myListener);
        walkButton.addActionListener(myListener);
        randomButton.addActionListener(myListener);
        gotoPitButton.addActionListener(myListener);
        gotoGoldButton.addActionListener(myListener);

		stayButton.setSelected(true);
		sel_wumpusMode = new String("STAY");
		getContentPane().setLayout(new FlowLayout());

	    picture = new JLabel(new ImageIcon("pit.gif"));
	    picture.setPreferredSize(new Dimension(50, 50));

		 j = new JPanel()
		 {
			 public void paintComponent(Graphics g)
			 {
				 super.paintComponent(g);
			 					stayButton.repaint();
			 					spinButton.repaint();
			 					walkButton.repaint();
			 					randomButton.repaint();
			 					gotoPitButton.repaint();
								gotoGoldButton.repaint();
								btn.repaint();
								btnHelp.repaint();
			}
		 };
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(j);
		j.setPreferredSize(new Dimension(300,200));
		j.setLayout(new FlowLayout());
		j.add(stayButton);
		j.add(spinButton);
		j.add(walkButton);
		j.add(randomButton);
		j.add(gotoPitButton);
		j.add(gotoGoldButton);

        btn = new JButton("OK");

        btnHelp = new JButton("Help");

        btn.addActionListener(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent e )
				{
					if(stayButton.isSelected())
					{
						sel_wumpusMode =new String("STAY") ;
					}
					else
					if(spinButton.isSelected())
					{
						sel_wumpusMode = new String("SPIN");
					}
					else
					if(walkButton.isSelected())
					{
						sel_wumpusMode = new String("WALK");
					}
					else
					if(randomButton.isSelected())
					{
						sel_wumpusMode = new String("RANDOM");
					}
					else
					if(gotoPitButton.isSelected())
					{
						sel_wumpusMode = new String("RANDOM");
						// Please change this once the algorithm is corrected
						// sel_wumpusMode = new String("GOTO_PIT");
					}
					else
					{
						sel_wumpusMode = new String("RANDOM");
						//please change this after a much efficient algorithm has been implemented
						// sel_wumpusMode = new String("GOTO_GOLD");
					}
					hide();
				}
			}
			);

        j.add(btn);
        j.add(btnHelp);

			panel = new JPanel()
			{
				public void paintComponent(Graphics g)
				{
					super.paintComponent(g);
					stayButton.repaint();
					spinButton.repaint();
					walkButton.repaint();
					randomButton.repaint();
					gotoPitButton.repaint();
					gotoGoldButton.repaint();
				}
			};

	   panel1 = new JPanel()
		{
			public void paintComponent(Graphics g)
			{
				super.paintComponent(g);
				btn.repaint();
				btnHelp.repaint();
			}
		};

		addWindowListener(
			new WindowAdapter()
			{
				public void windowActivated(WindowEvent e)
				{
					repaint();
				}
			}
			);
	}

	public void paint(Graphics g)
	{
		super.paint(g);


		j.repaint();

	}
    void setDimension(Dimension s)
	{
	   Random r = new Random(System.currentTimeMillis());
  	   j.setPreferredSize(new Dimension(390+r.nextInt()%10, 90 + r.nextInt()%10));
    }

    private  class RadioListener implements ActionListener
    {
        public void actionPerformed(ActionEvent e)
        {
	           /*
	           	Implement this to be able to select the mode of wumpus
	           	which is by default now set to "WALK"
	           */
	    }
	}
}

