import java.util.Random;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;

/**
Class Responsible for the behaviour of a Wumpus.
*/

public class Wumpus extends Actor
{


   private int mode;
   private Random randGen;
   private int action;
   private GridPoint origTarget;
   private GridPoint targetLoc;

 /**
 Constructor
 @param The object Type, the starting location, the orientation, the id number, the name of object and its mode

 */

   public Wumpus(int objType, int x, int y,int o, int idnumber, String name, String itsmode)
   {
    super(objType, x, y, o, idnumber, name);
    setMode(itsmode);
    randGen = new Random();
   }


   /**
   Returns the mode of a wumpus.
   See Class WumpusMode for various defined modes.
   */

   public int getMode()
   {
    return mode;
   }

  /**
  	This method sets a target location towards which the wumpus is headed

  */


   public void setTargetLoc(GridPoint gp)
   {
    origTarget = new GridPoint(gp.getPtX(),gp.getPtY());
    targetLoc = new GridPoint(gp.getPtX(),gp.getPtY());
   }
/**
	This method finds out the next action to be taken by the wumpus
*/
   public  void findAction(Environment env)
   {
    if(mode == WumpusMode.SPIN)
      {findSpinAction();}
    else if(mode == WumpusMode.STAY)
      {findStayAction();}
    else if(mode == WumpusMode.GOTO_GOLD)
      {findGotoAction(env);}
    else if(mode == WumpusMode.GOTO_PIT)
      {findGotoAction(env);}
    else if(mode == WumpusMode.WALK)
      {findWalkAction(env);}
    else
      {findRandomAction(env);}


   }
/**
	This method finds the spin action which is merely turning left.
*/
   public void findSpinAction()
   {
    action = Action.TURN_LEFT;
   }
/**
	This method finds the stay action which is merely sitting at the current location.
*/

   public void findStayAction()
   {
    action = Action.SIT;
   }

/**
	This method attempts to find the nearest available gold treasure that the wumpus
	can protect from agents or gold hunters
*/

   public GridPoint findNearestGold(Environment env)
   {
		Vector v = env.getAllGold();
		if(v==null)
		 return null;
		int len = v.size();
		if(len==0)
			return null;

		GridPoint org = (GridPoint) v.elementAt(0);
		double dist = org.getDistance(this.getLocation());
		GridPoint nearestGold = (GridPoint)org.clone();

		for(int i=1;i<len;i++)
		{
			GridPoint dGp = (GridPoint) v.elementAt(i);
			double dummy = dGp.getDistance(this.getLocation());
			if(dummy < dist)
				nearestGold = (GridPoint)dGp.clone();
		}

		return nearestGold;

   }

   private class PreferredGridPoint
   {
	   private GridPoint gp;
	   private int manDist;
	   private int distFromStart;
	   private int cost;

	   public PreferredGridPoint(int dfs,int md, GridPoint g)
	   {
		   distFromStart = dfs;
		   gp = (GridPoint)g.clone();
		   manDist = md;
		   calculateCost();
	   }

		public void calculateCost()
		{
			cost = distFromStart + manDist;
		}

		public int getCost()
		{
			return cost;
		}

		public void setDistFromStart(int s)
		{
			distFromStart = s;
		}

		public int getDistFromStart()
		{
			return distFromStart;
		}

	   public int getManhattanDist()
	   {
		   return manDist;
	   }

	   public GridPoint getGridPoint()
	   {
		   return gp;
	   }

	   public String toString()
	   {
		   return new String("Cost = " + cost);
	   }
   }


   public void findGotoAction(Environment env)
   {

	/*
	 Lets implement the A* algorithm
	 1. Find the most nearest grid location that has gold and set the
	    target wumpus location based on the shortest crow flight distance
	 2. Once that has been set put an object in the queue that has cost
	    calculated as zero+distance
	 3. Sort it in asc order
	 4. For the element at the top, take that and add few other points and
	 5. Repeat 3,4 until a gold location is found.
	*/

	Vector queue = new Vector();
	GridPoint x = (GridPoint) findNearestGold(env);
	if (x==null)
	{
		action = Action.SIT;
		return;
	}
	targetLoc = (GridPoint)x.clone();
	System.out.println(targetLoc);
	System.out.println(targetLoc.getManhattanDist(this.getLocation()));

	int mhd = targetLoc.getManhattanDist(this.getLocation());
	queue.add(new PreferredGridPoint(0,mhd, this.getLocation()));

        try
        {
			System.in.read();
		}
		catch(Exception i)
		{

		}

	if(env.isGoldAt(this.getLocation()))
	{
 		action = Action.SIT;
		return;
	}

	Hashtable hTable = new Hashtable();

    GridPoint wumpLoc = this.getLocation();



    boolean foundGold = false;
    GridPoint gp = null;

    Enumeration e;
    PreferredGridPoint p=null;
    while(!queue.isEmpty() && !foundGold)
    {

		int len = queue.size();
		p = (PreferredGridPoint) queue.elementAt(0);
		int cost = p.getCost();
		PreferredGridPoint dummy;
		int rmElementAt=0;
		for(int i=1;i<len;i++)
		{
			dummy = (PreferredGridPoint)queue.elementAt(i);
			if(dummy.getCost()<cost)
			{
				p = dummy;
				cost = dummy.getCost();
				rmElementAt = i;
			}
		}
		hTable.put(p.getGridPoint(), this.getLocation());

		//System.out.println("This is the next one chosen " + p.getGridPoint());
		queue.removeElementAt(rmElementAt);

		Vector adjGp = new Vector();


		if(env.isValid(p.getGridPoint().getNorth()))
			adjGp.add(p.getGridPoint().getNorth());
		if(env.isValid(p.getGridPoint().getEast()))
			adjGp.add(p.getGridPoint().getEast());
		if(env.isValid(p.getGridPoint().getSouth()))
			adjGp.add(p.getGridPoint().getSouth());
		if(env.isValid(p.getGridPoint().getWest()))
			adjGp.add(p.getGridPoint().getWest());

		System.out.println(adjGp);

		if(adjGp.size()==0)
		{
			action = Action.SIT;
			return;
		}

		int length = adjGp.size();
		for(int i=0; i<length; i++)
		{
			gp = (GridPoint) adjGp.elementAt(i);

				if(!env.isObstacleAt(gp.getPtX(), gp.getPtY()))
				{


					queue.add(new PreferredGridPoint(p.getDistFromStart()+1,gp.getManhattanDist(targetLoc),gp));
					//System.out.println("Checking if " + gp +" has gold");
                    if(env.isGoldAt(gp))
                    {
						foundGold = true;
						break;
					}
				}
		}
	}
		hTable.put(gp, this.getLocation());

		System.out.println(hTable);
		System.out.println(gp);

		GridPoint hopTo = (GridPoint)hTable.get(gp);

		while(!hopTo.equals(p.getGridPoint()))
		{
			gp = hopTo;
			hopTo = (GridPoint) hTable.get(gp);
			System.out.println(gp);
		}
	targetLoc = (GridPoint)gp.clone();

     if(this.getOrientation() == Directions.EAST)
     {
		if(targetLoc.getPtX()<= wumpLoc.getPtX())
		{
			if(targetLoc.getPtY() < wumpLoc.getPtY())
				action = Action.TURN_LEFT;
			else
				if(targetLoc.getPtY() > wumpLoc.getPtY())
					action = Action.TURN_RIGHT;
				else
					action = Action.SIT;

		}
		 else
 		     action = Action.GOFORWARD;
 	 }
     if(this.getOrientation() == Directions.SOUTH)
     {
		if(targetLoc.getPtY()<= wumpLoc.getPtY())
		{
			if(targetLoc.getPtX() > wumpLoc.getPtX())
				action = Action.TURN_LEFT;
			else
				if(targetLoc.getPtX() < wumpLoc.getPtX())
					action = Action.TURN_RIGHT;
				else
					action = Action.SIT;

		}
		 else
 		     action = Action.GOFORWARD;
 	 }
     if(this.getOrientation() == Directions.WEST)
     {
		if(targetLoc.getPtX()>= wumpLoc.getPtX())
		{
			if(targetLoc.getPtY() > wumpLoc.getPtY())
				action = Action.TURN_LEFT;
			else
				if(targetLoc.getPtY() < wumpLoc.getPtY())
					action = Action.TURN_RIGHT;
				else
					action = Action.SIT;

		}
		 else
 		     action = Action.GOFORWARD;
 	 }
     if(this.getOrientation() == Directions.NORTH)
     {
		if(targetLoc.getPtY()>= wumpLoc.getPtY())
		{
			if(targetLoc.getPtX() < wumpLoc.getPtX())
				action = Action.TURN_LEFT;
			else
				if(targetLoc.getPtX() > wumpLoc.getPtX())
					action = Action.TURN_RIGHT;
				else
					action = Action.SIT;
		}
		 else
 		     action = Action.GOFORWARD;
 	 }




   }

   public void findWalkAction(Environment env)
   {
    GridPoint wumpLoc = this.getLocation();
    GridPoint nextLoc = new GridPoint(wumpLoc.getPtX(),wumpLoc.getPtY());

    moveforward(nextLoc);
    if(env.isValid(nextLoc))
    {
     action = Action.GOFORWARD;
     System.out.println("Going forward");
    }
    else
    {
     action = Action.TURN_LEFT;
     System.out.println("Turn left");
    }
   }

   public void findRandomAction(Environment env)
   {

    int randnum = randGen.nextInt(4);
    GridPoint wumpLoc = this.getLocation();
    GridPoint nextLoc = new GridPoint(wumpLoc.getPtX(),wumpLoc.getPtY());

    moveforward(nextLoc);

    if(randnum == 0 && (env.isValid(nextLoc)))
     action = Action.GOFORWARD;
    else if(randnum == 1)
     action = Action.TURN_LEFT;
    else if(randnum == 2)
     action = Action.TURN_RIGHT;
    else
     action = Action.SIT;
   }


   public GridPoint findNewLoc()
   {
    GridPoint newWumpusLoc = (GridPoint)this.getLocation().clone();

    if(action == Action.GOFORWARD)
    {
     moveforward(newWumpusLoc);
    }

    return newWumpusLoc;
   }



   public void act(boolean actOrNot)
   {
    if(actOrNot)
    {
     if(action == Action.GOFORWARD)
       this.move();
     else if(action == Action.TURN_LEFT)
       this.turnLeft();
     else if(action == Action.TURN_RIGHT)
       this.turnRight();
    }
    return;
   }

   public void moveforward(GridPoint wumploc)
   {
    int x, y;

    x = wumploc.getPtX();
    y = wumploc.getPtY();

    if(this.getOrientation() == Directions.WEST)
    {
      wumploc.advanceLeft();
    }
    else if(this.getOrientation() ==Directions.EAST)
    {
      wumploc.advanceRight();
    }
    else if(this.getOrientation() == Directions.NORTH)
    {
      wumploc.advanceUp();
    }
    else if(this.getOrientation() == Directions.SOUTH)
    {
      wumploc.advanceDown();
    }

   }
/**
	This method sets a mode for the wumpus given a String parameter
	@param String representing a mode of Wumpus
*/
   private void setMode(String somemode)
   {
     if(somemode.equalsIgnoreCase("STAY"))
      mode = WumpusMode.STAY;
     else if(somemode.equalsIgnoreCase("SPIN"))
      mode = WumpusMode.SPIN;
     else if(somemode.equalsIgnoreCase("WALK"))
      mode = WumpusMode.WALK;
     else if(somemode.equalsIgnoreCase("GOTO_GOLD"))
      mode = WumpusMode.GOTO_GOLD;
     else if(somemode.equalsIgnoreCase("GOTO_PIT"))
      mode = WumpusMode.GOTO_PIT;
     else
      mode = WumpusMode.RANDOM;
   }

}
